// common in different pathfinding algorithms
var DataArray[SMALLMAPX][SMALLMAPY];  	// to convert tile values to accessibility and move cost values
var sx = 40;									// "start" tile position
var sy = 40;
var fx = 50;									// "finish" tile position 
var fy = 50;
var tx, ty; 	      						// temporary tile position
var found;					   				// found: destination is reached
var maxpath;									// to maximize cycle time and path length, to avoid infinite cycles and too long calculations
var pathlength = 0;							// show actual path length
// the resulting path array holding x;y coordinate values of path steps
var pathX[MAXPATHLENGTH], pathY[MAXPATHLENGTH]; 			
// path directions in each step 
var pathDir[MAXPATHLENGTH];
var cycle_time = 0;							// to display cycle number in the end

// A*
var astar_clist[SMALLMAPX][SMALLMAPY];			
var astar_olist[SMALLMAPX][SMALLMAPY];		
// values needed for distance estimation and tracking
var astar_F[SMALLMAPX][SMALLMAPY];
var astar_G[SMALLMAPX][SMALLMAPY];
var astar_H[SMALLMAPX][SMALLMAPY];
// storing previous path element of each node - it cannot be known during the search where the final path goes
var astar_parent_x[SMALLMAPX][SMALLMAPY];
var astar_parent_y[SMALLMAPX][SMALLMAPY];

// pathfinder user settings
var drawalltiles = 1;						// what to display after path search, 1: draw all the possible paths , 0: draw only the path dots
var path_type = 1;							// 1 direction map, 2 A* manhattan, 3 A* euclidian, 4 A* diagonal shortcut 
var heur_value = 10;
var total_move_cost = 0;
var turnlimit = 0;
var depthfirst = 0;

// path search time needed
float time_1, time_2;
float time_elapsed;
STRING* timer_value = "0.000";

// representing the paths
ENTITY* ArrowArray[SMALLMAPSIZE];
ENTITY* PathArray[MAXPATHLENGTH];
STRING* path_dot = "path_dot.tga";	// pathfinder sprite

/////////////////////////////////////////

// path related common functions
function EndOfPathfinding();
function Path_Cost_Calculation();

///////////////////////////////////////////

PANEL* work_in_progress =
{
	pos_x = 924; 
	pos_y = 300;
	bmap = "work.bmp";	
	layer = 1;
}

PANEL* path_error =
{
	pos_x = 924; 
	pos_y = 300;
	bmap = "error.bmp";	
	layer = 3;
}

PANEL* path_found =
{
	flags = SHADOW;
	pos_x = 924; 
	pos_y = 300;
	bmap = "found.bmp";	
	layer = 2;
}

TEXT* timer_txt =	
{
  layer = 1;
  pos_x = 512;
  pos_y = 650;
  strings = 1;
  font = "Arial#36b";
  string ("Last search time interval [sec]: " , timer_value);
  flags = CENTER_X | OUTLINE | SHOW;
} 

TEXT* wait_txt =	
{
  layer = 1;
  pos_x = 512;
  pos_y = 600;
  blue=0;
  green=0;
  red=255;
  strings = 1;
  font = "Arial#36b";
  string ("Please wait...");
  flags = CENTER_X | OUTLINE;
} 

//////////////////////////////////////////

#include "objecthandling.c"
#include "panelhandling.c"

/////////////////////////////////////////
// common after-pathfinding function calls

// reset things after pathfinding
function EndOfPathfinding()
{
	BuildMode = 0; 
	ChangeMode();
	reset(wait_txt,SHOW);
}

// move cost calculation based on actual path and terrain tile values
function Path_Cost_Calculation()
{
	var i,j,k;
	// Tile value conversion to pathfinding move cost values stored by DataArray 
	// it is done in A* but in the same way, but in DirMap it is different, so should be done now
	for(i=0 ; i<GridX ; i++)
		for(j=0 ; j<GridY ; j++)
		{						
			// water or other forbidden area - no access
			if (Tiles[i][j]%HEIGHT_OFFSET == 1) { DataArray[i][j] = 0; }
			
			// grass = normal terrain
			else if (Tiles[i][j]%HEIGHT_OFFSET == 0) { DataArray[i][j] = GRASS_COST; }
			// stone road
			else if (Tiles[i][j]%HEIGHT_OFFSET == 2) { DataArray[i][j] = STONE_COST; }
			// dirt road
			else if (Tiles[i][j]%HEIGHT_OFFSET == 3) { DataArray[i][j] = DIRT_COST; }
			// woods
			else if (Tiles[i][j]%HEIGHT_OFFSET == 4) { DataArray[i][j] = WOODS_COST; }
			// swamp
			else if (Tiles[i][j]%HEIGHT_OFFSET == 7) { DataArray[i][j] = SWAMP_COST; }
			
			// ladder
			else if (Tiles[i][j]%HEIGHT_OFFSET == MODELS+24) { DataArray[i][j] = LADDER_COST; }
			// stairs
			else if (Tiles[i][j]%HEIGHT_OFFSET == MODELS+20) { DataArray[i][j] = STAIRS_COST; }
			// on wall
			else if (Tiles[i][j]%HEIGHT_OFFSET == MODELS+0) { DataArray[i][j] = ONWALL_COST; }
			// in tower
			else if (Tiles[i][j]%HEIGHT_OFFSET == MODELS+1) { DataArray[i][j] = ONWALL_COST; }
			
			// if on ground, add slope movecost - mountain is double as hill
			if (Tiles[i][j]%HEIGHT_OFFSET<MODELS) { DataArray[i][j] += integer(Tiles[i][j]/HEIGHT_OFFSET)*HILL_COST; }
		}  
		
	// reset value
	total_move_cost = 0;	
	// add tile values
	for(k=0 ; k<pathlength ; k++)
	{
		i = pathX[k];
		j = pathY[k];
		total_move_cost += DataArray[i][j];		
	}
}
