
// tile system: X*Y tiles (defined in main script), 64x64 quant per tile (or 32 but it needs scaling)
var TileSize = GRIDSIZE;
var GridX = SMALLMAPX; 
var GridY = SMALLMAPY; 

// 2.5 dimensional tile array
var Tiles[SMALLMAPX][SMALLMAPY];		// stores terrain surface data 
var TilesZ[SMALLMAPX][SMALLMAPY];	// z level data for 2.5dimensional pathfinding, it means top height of a walkable building e.g. wall/tower

// drawing info - should be measured after map loaded - to make faster sprite creation
var TilesMin[SMALLMAPX][SMALLMAPY];
var TilesMax[SMALLMAPX][SMALLMAPY];
var TilesMid[SMALLMAPX][SMALLMAPY];

// camera_control
var ZoomSpeed = 15;
var RotateSpeed = 7.5;
var MoveSpeed = 100;
var ZoomDownLimit = 225;
var ZoomUpLimit = 2500;
var XMax =  SMALLMAPX*GRIDSIZE/2;
var XMin = -SMALLMAPX*GRIDSIZE/2;
var YMax =  SMALLMAPY*GRIDSIZE/2;
var YMin = -SMALLMAPY*GRIDSIZE/2;

// camera data
STRING* CameraModel = "camera_cube.mdl";  // any simple modell
ENTITY* CameraObject;							// that is followed by the camera
var CamZoom = 300;								// camera height 1000 ok
var CamAngle = 270-45; 							// rotation of camera around camera object: 270 = watch from South i.e. watch to North
var CamHeight = 200;								// height of camera object - should be constant but over surface, or constantly over surface

// mouse data
BMAP* mousearrow = "mousearrow.tga";

// active tile data
ENTITY* ActiveTileSquare;												// the square entity
STRING* square1 = "square1.tga";      								// build square
STRING* square2 = "square2.tga";										// endpoint square 
STRING* square3 = "square3.tga";										// wait square 
var square_coord_x , square_coord_y , square_coord_z = 0;	// square entity coordinates
var square_x , square_y = 0;											// square entity grid x,y position
var square_value = 0;													// local value of Tiles
var square_valueZ = 0;													// local value of TilesZ
var draw_size = 1;														// some object can be larger than 1 tile, e.g. the tower is 3=3x3

// path- and ActiveTileSquare-sprite height offset values
var RoadLevel = 0.2;
var PathLevel = 0.5;
var SquareLevel = 1;

// terrain
ENTITY* terrain_entity;

// player and flag models
STRING* playermodel = "pguard3.mdl";
//STRING* destsprite = "flag2.tga";
STRING* destmodel = "flag.mdl";
ENTITY* destobject;

var BuildMode = 0;							// defines what to do
var temp_tx, temp_ty;						// global temporary variables showing tile position - used in object actions and functions 

//////////////////////////////////////////////
// iso sys function pre-declarations

// initialization
function Interface_Init();

// camera
action SetCamera();   
function TestLimits();
function Camera_Init();

// tile system
function TileSys_Init();

// mouse camera operations
function Mouse_Init();

// ActiveTilePos functions
function Mouse3D();
function Mouse3D2();
function SnapIt();
function ShowPosition();	// important unit conversions
function Square_Init();
action Square();
// mode changes
function ChangeMode();

// level detection
function PosInfo3D(tempx, tempy);
function PosInfo3D2(tempx, tempy);
function TileLowHighPosInfo3D(tempx, tempy, lohi);
function MapZData();
function SetHeightOffset(i,j);

// unit conversion
function TileToCoord( xytile , grid );

// player settings
function AlignPlayer();
action CreatePlayer();
function Player_Init();

// destination flag settings
function AlignDestFlag();
action CreateDestFlag();
function Destination_Init();

////////////////////////////////////

#include "commonpath.c"
#include "objecthandling.c"
#include "panelhandling.c"
#include "pathdirmap.c"
#include "pathastar.c"
#include "pathastar_turn.c"

////////////////////////////////////////////////


// start of the system
function Interface_Init()
{			
	// necessary function calls 
	TileSys_Init();		// tile value nulling
	wait(5);
	Camera_Init(); 		// CameraObject entity creation , to be followed by camera
	wait(2);
	Mouse_Init();			// mouse start
	Square_Init();   		// ActiveTileSquare creation and settings
	
	Destination_Init();	// create destination flag model
	Player_Init();			// create player model
	ChangeMode();			// set the proper skin of ActiveTileSquare 
		
	// MAP PRE-PROCESSING
	// creating map height data array for each tile, it makes much faster model and sprite placement
	// needed: max (for sprites) , min (for buildings), midpoint (for moving models)
	//
	// but also can be used for converting these values to tile data - instead of setting it tile by tile - in the following way: 
	// under 0 it is water; high min and max difference can mean hill/mountaing slope that results in movepoint penalty, 
	// or in extreme case no access area - can be different for different type of player units (infantry, cavalry, artillery)
	//
	// moreover, because a hilly terrain would result in vertical offset of sprites thus ugly look, 
	// sprites should be avoided on non-flat surfaces, 
	// or used only for assigning tile values but removed in game mode 
	// (e.g. roads can be assigned by sprites, but should be represented by terrain skin texture)
	MapZData();
	
	// panels settings
	button_state(info_panel_1,1,ON); 
	button_state(info_panel_2,7,ON); 
		
	// user function calls
	wait(5);
	
	on_space = ClearMap;  // clear obstacles from map and delete tile values

	// object placement
	on_mouse_left = PlaceObject;
	// object moving/deleting
	on_mouse_right = PlaceObject2;	
	
	while(1)
	{
		// global variable sets the water sprite animation speed
		water_anim += 0.25 * time_step;
	   water_anim %= 7;
		
		if (key_any)
		  if (BuildMode!=255)		// path showing state
		  {
			// build / delete mode
			if(key_1) {BuildMode = 1;}		// water placement - sprite creation
			if(key_2) {BuildMode = 2;}		// stone road placement - sprite creation
			if(key_3) {BuildMode = 3;}		// dirt road placement - sprite creation
			if(key_4) {BuildMode = 4;}		// wall placement - 1-tile model creation
			if(key_5) {BuildMode = 5;}		// tower placement - multi-tile model creation
			if(key_6) {BuildMode = 6;}		// tree placement - 1-tile model creation
				
			if(key_9) {BuildMode = 99;}	// ladder placement - 1-tile model creation
			
			// +90 pan
			if(key_r) 
			{
				while (key_r) {wait (1);}
				RotateEntity(square_x,square_y);				
			}		
			
			// change path endpoints mode
			if(key_0) {BuildMode = 0;}		// replace path endpoints mode
			
			// update buttons
			if (BuildMode>0) button_state( info_panel_2, BuildMode , ON );
			else button_state( info_panel_2, 7 , ON );
			
			// toggle path representation mode
			if (key_f12)
			{
				while (key_f12) {wait (1);}
				
				drawalltiles = 1 - drawalltiles;  // 0 or 1
				// update button
				button_state( info_panel_1,6, (1-drawalltiles) );
			}		
			if (key_f10)
			{
				while (key_f10) {wait (1);}
				
				turnlimit = 1 - turnlimit;  // 0 or 1
				// update button
				button_state( info_panel_1,8, (turnlimit) );
			}	
			if (key_f6)
			{
				while (key_f6) {wait (1);}
				
				depthfirst = 1 - depthfirst;  // 0 or 1
				// update button
				button_state( info_panel_1,10, (depthfirst) );
			}
			
			// start path calculations
			if(key_f1) // direction map / breadth first
				{
					set(wait_txt,SHOW);
					// update panel menu values
					path_type = 1;
					button_state(info_panel_1,1,ON); 
					change_algorythm(1); // call pathfinder with default heuristic value
					BuildMode = 255;
					PathFind00();
				}
			else if(key_f2) 	// A*
				{
					set(wait_txt,SHOW);
					// update panel menu values
					path_type = 2;
					button_state(info_panel_1,2,ON); 
					change_algorythm(2); // call pathfinder with default heuristic value
					BuildMode = 255;
					if (turnlimit) PathFind02(1);
					else PathFind01(1);
				}
			else if(key_f3) 
				{
					set(wait_txt,SHOW);
					// update panel menu values
					path_type = 3;
					button_state(info_panel_1,3,ON);
					change_algorythm(3); // call pathfinder with default heuristic value
					BuildMode = 255;
					if (turnlimit) PathFind02(2);
					else PathFind01(2);
				}
			else if(key_f4) 
				{
					set(wait_txt,SHOW);
					// update panel menu values
					path_type = 4;
					button_state(info_panel_1,4,ON);
					change_algorythm(4); // call pathfinder with default heuristic value
					BuildMode = 255;
					if (turnlimit) PathFind02(3);
					else PathFind01(3);
				}
			else if(key_f5) 
				{
					set(wait_txt,SHOW);
					// update panel menu values
					path_type = 0;
					button_state(info_panel_1,9,ON);
					change_algorythm(9); // call pathfinder with default heuristic value
					BuildMode = 255;
					PathFind01(0);
				}
				
			ChangeMode();			// change ActiveTileSquare's bitmap due to BuildMode		
			}		
		wait(1);
	}	
}

/////////////////////////////////////////////////
// camera handling functions

// make it undetectable by c_trace calls
action SetCamera()
{
	CameraObject = me;
	my.ENTITY_TYPE = 103;
//	my.z = CamHeight;
	set( me , PASSABLE | INVISIBLE | FLAG2 );
}

// set min and max coordinates of camera object
function TestLimits()
{
	CameraObject.x = maxv(minv(CameraObject.x, XMax), XMin);
	CameraObject.y = maxv(minv(CameraObject.y, YMax), YMin);
}

// place camera object entity to be followed by camera
function Camera_Init()
{  	
	ent_create( CameraModel, vector(-1600,-1600,CamHeight), SetCamera);
	
	while(1)
	{                
		camera.z = CamZoom;
		camera.x = CameraObject.x + CamZoom * cos(CamAngle); 
		camera.y = CameraObject.y + CamZoom * sin(CamAngle); 
		
		var temp;
		
		vec_set(temp,CameraObject.x);
		vec_sub(temp,camera.x);
		vec_to_angle(camera.pan,temp);              
		CamAngle %= 360;           
		
		wait(1);
	}
}

//////////////////////////////////////////////////
// tile value nulling
function TileSys_Init()
{
	var i,j;
	for (j=0 ; j<GridX ; j++)
		for (i=0 ; i<GridY ; i++)
		{
			Tiles[j][i]=0;
			TilesZ[j][i]=0;
		}
}

///////////////////////////////////
// mouse related functions

// mouse initialization and camera movements
function Mouse_Init()
{
	mouse_map = mousearrow;
	mouse_mode = 1;
	while (mouse_mode > 0)
	{
		vec_set(mouse_pos , mouse_cursor);                       
		
		// camera movements by mouse
		// forward
		if (mouse_pos.y <= 2)
		{
			CameraObject.y += sin(camera.pan) * MoveSpeed * time_step + (key_shift * sin(camera.pan) * MoveSpeed * time_step);
			CameraObject.x += cos(camera.pan) * MoveSpeed * time_step + (key_shift * cos(camera.pan) * MoveSpeed * time_step);
			TestLimits();
		}
		// back
		if (mouse_pos.y >= screen_size.y -2)
		{
			CameraObject.y -= sin(camera.pan) * MoveSpeed * time_step + (key_shift * sin(camera.pan) * MoveSpeed * time_step);
			CameraObject.x -= cos(camera.pan) * MoveSpeed * time_step + (key_shift * cos(camera.pan) * MoveSpeed * time_step);
			TestLimits();
		}
		// left
		if ( (mouse_pos.x <= 2) && (mouse_pos.y > screen_size.y/2))
		{
			CameraObject.y += cos(camera.pan) * MoveSpeed * time_step + (key_shift * cos(camera.pan) * MoveSpeed * time_step);
			CameraObject.x -= sin(camera.pan) * MoveSpeed * time_step + (key_shift * sin(camera.pan) * MoveSpeed * time_step);
			TestLimits();
		}
		// right
		if ((mouse_pos.x >= screen_size.x -2) && (mouse_pos.y > screen_size.y/2))
		{
			CameraObject.y -= cos(camera.pan) * MoveSpeed * time_step + (key_shift * cos(camera.pan) * MoveSpeed * time_step);
			CameraObject.x += sin(camera.pan) * MoveSpeed * time_step + (key_shift * sin(camera.pan) * MoveSpeed * time_step);
			TestLimits();
		}
		// rotate left
		if ( (mouse_pos.x <= 2) && (mouse_pos.y <= screen_size.y/2))
		{
			CamAngle += RotateSpeed * time_step;                        
		}
		// rotate right
		if ((mouse_pos.x >= screen_size.x -2) && (mouse_pos.y <= screen_size.y/2))
		{
			CamAngle -= RotateSpeed * time_step;                      
		}
		// zoom
		if (!(mickey.z==0)) 
		{
			CamZoom += ZoomSpeed * 0.05 * mickey.z ;
			if (CamZoom < ZoomDownLimit)
			{
				CamZoom = ZoomDownLimit;
			}	
			if (CamZoom > ZoomUpLimit)
			{
				CamZoom = ZoomUpLimit;
			}
		}
		
		wait(1);
	}
}

//////////////////////////////////
// ActiveTileSquare related functions

// get mouse position on terrain surface
function Mouse3D()
{
	VECTOR pos1, pos2;

	vec_set( pos1 , vector(mouse_pos.x,mouse_pos.y,0) );
	vec_for_screen (pos1, camera);
	
	vec_set( pos2 , vector(mouse_pos.x,mouse_pos.y,20000) ); // should be a big value       		
	vec_for_screen (pos2, camera);

	c_trace (pos1.x, pos2.x, IGNORE_SPRITES | IGNORE_PASSABLE | IGNORE_MODELS | IGNORE_MAPS ); 
	// if there is a HIT_TARGET event, now "target" and "hit.x" hold the coordinates of the hit point 
	// ignoring tile sprites, models, map-entities and other passable entities including passable terrains (water)      
}

// get mouse position on terrain and some special model/map-entity surfaces
function Mouse3D2()
{
	VECTOR pos1, pos2;

	vec_set( pos1 , vector(mouse_pos.x,mouse_pos.y,0) );
	vec_for_screen (pos1, camera);
	
	vec_set( pos2 , vector(mouse_pos.x,mouse_pos.y,20000) ); // should be a big value       		
	vec_for_screen (pos2, camera);

	c_trace (pos1.x, pos2.x, IGNORE_SPRITES | IGNORE_FLAG2); 
	// if there is a HIT_TARGET event, now "target" and "hit.x" hold the coordinates of the hit point 
	// ignoring tile sprites, and flag2 entities (those have no walkable surface - for further development)      
}


// snapping to tile midpoint, within grid limits
function SnapIt()
{  
	// x-axis snapping
	if (my.x < 0)
	{
		// snap
		my.x = TileSize * (integer(my.x / TileSize)) - (TileSize / 2);		
		// limit   
		if (my.x < -((GridX / 2) * TileSize - (TileSize / 2)))
		{
			my.x = -((GridX / 2) * TileSize - (TileSize / 2));
		}
	}
	else if (my.x >= 0)
	{
		// snap
		my.x = TileSize * (integer(my.x / TileSize)) + (TileSize / 2);
		// limit
		if (my.x > ((GridX / 2) * TileSize - (TileSize / 2)))
		{
			my.x = ((GridX / 2) * TileSize - (TileSize / 2));
		}
	}       
	
	// y-axis snapping
	if (my.y < 0)
	{
		// snap
		my.y = TileSize * (integer(my.y / TileSize)) - (TileSize / 2);
		// limit
		if (my.y < -((GridY / 2) * TileSize - (TileSize / 2)))
		{
			my.y = -((GridY / 2) * TileSize - (TileSize / 2));
		}
	}
	else if (my.y >= 0)
	{
		// snap
		my.y = TileSize * (integer(my.y / TileSize)) + (TileSize / 2);
		// limit
		if (my.y > ((GridY / 2) * TileSize - (TileSize / 2)))
		{
			my.y = ((GridY / 2) * TileSize - (TileSize / 2));
		}
	}  
}

// determine ActiveTileSquare x,y tile position and value, for further calculations and display
function ShowPosition()
{  	
	square_coord_x = ActiveTileSquare.x;
	square_coord_y = ActiveTileSquare.y;
	square_coord_z = ActiveTileSquare.z;
	
	square_x = integer ( (ActiveTileSquare.x + 1 * GridX / 2 * TileSize)/TileSize );
	square_y = integer ( (ActiveTileSquare.y + 1 * GridY / 2 * TileSize)/TileSize );

	//to display the value stored in the actual node
	square_value = Tiles[square_x][square_y];
	square_valueZ = TilesZ[square_x][square_y];
}

// set ActiveTileSquare place and properties, and refresh in each frame
action Square()
{          
	// give an entity name to square sprite eablig tracking x,y coordinates
	ActiveTileSquare = me;
	
	my.ENTITY_TYPE = 102;
	set( me , PASSABLE );			// passable to avoid c_tracing, | ZNEAR render over other entities
	my.scale_x = TileSize/GRIDSIZE;
	my.scale_y = TileSize/GRIDSIZE;
	// horizintal alignment
	vec_set( my.pan , vector(270,90,0) ); 
	
	while(1)
	{         	                
		if (BuildMode == 0) Mouse3D2();	// call c_tracing to get 3D position on any surface  								
		if (BuildMode != 0) Mouse3D(); 	// call c_tracing to get 3D position on terrain	
		
		// move square only if mouse points within terrain - else it remains in previous position		
		if (HIT_TARGET) 					// without this check it would point to Node (0,0) if mouse points out of terrain
		{
			my.x=hit.x;
			my.y=hit.y;
			
			SnapIt();  						// snap to middle of tile
			ShowPosition();
			
			if ( (BuildMode == 0) && (Tiles[square_x][square_y]%HEIGHT_OFFSET>=MODELS) )
				{ my.z = TilesMin[square_x][square_y] + SquareLevel + TilesZ[square_x][square_y]*TileSize; }
			else 
				{ my.z = TilesMax[square_x][square_y] + SquareLevel + RoadLevel; }			
		}		
		
		wait(1);
	}
}

// create ActiveTileSquare anywhere
function Square_Init()
{
	ent_create( square1, nullvector, Square);	
}

// set proper ActiveTileSquare skin showing actual buildmode
function ChangeMode()
{ 
	if (BuildMode == 0)
	{
		ent_morph(ActiveTileSquare, square2);	// blue - modify path
	}	
	else if (BuildMode == 255)
	{
		ent_morph(ActiveTileSquare, square3);	// red - wait
	}
	else
	{
		ent_morph(ActiveTileSquare, square1);	// green - build
	}	
}

/////////////////////////////////////////
// different terrain level detection and coordinate conversion functions

// using c-trace for getting tile terrain surface information at any x;y coordinate position
function PosInfo3D(tempx, tempy)
{
	VECTOR pos1, pos2;

	vec_set( pos1 , vector(tempx, tempy, -1000) );
	vec_set( pos2 , vector(tempx, tempy, 1000) ); // should be a big value       		
	
	c_trace (pos2, pos1, IGNORE_SPRITES | IGNORE_MODELS | IGNORE_MAPS | IGNORE_PASSABLE );  
	
	// if (HIT_TARGET) return(hit.z);
}

// using c-trace for getting tile building surface information at any x;y coordinate position
function PosInfo3D2(tempx, tempy)
{
	VECTOR pos1, pos2;

	vec_set( pos1 , vector(tempx, tempy, -1000) );
	vec_set( pos2 , vector(tempx, tempy, 1000) ); // should be a big value       		
	
	c_trace (pos2, pos1, IGNORE_SPRITES | IGNORE_FLAG2 | IGNORE_PASSABLE);    
	
	// if (HIT_TARGET) return(hit.z);
}

//// getting 0-lowest/1-highest z value of a tile having tempx,tempy coordinates in the middle - so it can be called only after Snapit()
function TileLowHighPosInfo3D(tempx, tempy, lohi)
{
	var heightvalue[4];
	var temp, i;
	
	// NW 0
	PosInfo3D(tempx - TileSize/2, tempy + TileSize/2);    
	if (HIT_TARGET) heightvalue[0] = hit.z; 
	// NE 1
	PosInfo3D(tempx + TileSize/2, tempy + TileSize/2);    
	if (HIT_TARGET) heightvalue[1] = hit.z; 
	// SW 2
	PosInfo3D(tempx - TileSize/2, tempy - TileSize/2);    
	if (HIT_TARGET) heightvalue[2] = hit.z; 
	// SE 3
	PosInfo3D(tempx + TileSize/2, tempy - TileSize/2);   
	if (HIT_TARGET) heightvalue[3] = hit.z; 
	
	if (lohi==0) {temp = 1000;}  // should be high to get the lowest 
	else {temp = -1000;}			// should be low to get the highest
	
	for(i=0;i<4;i++)
	{
		if (lohi==0)
		{
			if ( heightvalue[i] < temp ) temp = heightvalue[i];	
		}
		else
		{
			if ( heightvalue[i] > temp ) temp = heightvalue[i];		
		}
	}
	// give the highest z value among the 4 samples
	return(temp);
}

// determine x/y coordinate of a tile by reverse calculation from: square_x = integer ( (ActiveTileSquare.x + 1 * GridX / 2 * TileSize)/TileSize );
function TileToCoord( xytile , grid )
{	
	var temp = ( xytile * TileSize ) - 1 * grid / 2 * TileSize +1;	// +1 isneeded for correct snapping
	return( temp );
}

// measure map height data in each tile
function MapZData()
{
	var i,j,tempx,tempy,tempa,tempb;
	
	// bottomleft tile midpoint coordinates
	tempx = - GridX/2*TileSize + TileSize/2;
	tempy = - GridY/2*TileSize + TileSize/2;
	
	for (j=0;j<GridY;j++)
		for (i=0;i<GridX;i++)	
		{
			tempa = tempx + TileSize*i;
			tempb = tempy + TileSize*j;
			
			TilesMin[i][j] = TileLowHighPosInfo3D(tempa, tempb, 0);
			TilesMax[i][j] = TileLowHighPosInfo3D(tempa, tempb, 1);
			
			PosInfo3D(tempa, tempb);
			if (HIT_TARGET) TilesMid[i][j] = hit.z;
			
			SetHeightOffset(i,j);
		}
}

function SetHeightOffset(i,j)
{
	// convert depth data to tile value
	if (TilesMin[i][j]<-TileSize) 
	{ 
		Tiles[i][j]=1; 
		return;	
	}												// water if between 0 and 1 tile depth
	else if ((TilesMin[i][j]<=0) && (TilesMin[i][j]>=-TileSize)) 
	{ 
		Tiles[i][j]=7;
		return; 
	}	// swamp if more than 1 tile deep
	
	// convert slope data to tile value
	// hill
	if ((abs(TilesMax[i][j]-TilesMin[i][j])>(0.25*TileSize)) && (abs(TilesMax[i][j]-TilesMin[i][j])<=0.75*TileSize))
	{	Tiles[i][j]+= HEIGHT_OFFSET; }																						
	// mount
	else if ((abs(TilesMax[i][j]-TilesMin[i][j])>0.75*TileSize) && (abs(TilesMax[i][j]-TilesMin[i][j])<=1.25*TileSize))
	{	Tiles[i][j]+= 2*HEIGHT_OFFSET; }																
	// abyss
	else if (abs(TilesMax[i][j]-TilesMin[i][j])>1.25*TileSize)
	{	Tiles[i][j] = 1; }
}

//////////////////////////////////////
// setting entities showing path start and end positions

// start point entity
function AlignPlayer()
{
	my.ANIMATION = 0;
	my.XTILE_NUMBER = sx;
	my.YTILE_NUMBER = sy;
	my.AREA_SIZE = 1;	
	my.ENTITY_TYPE = 100;
	player = me;	
		
	// scale to actual tilesize
	my.scale_x = TileSize/GRIDSIZE;
	my.scale_y = TileSize/GRIDSIZE;
	my.scale_z = TileSize/GRIDSIZE;
//	// set z
//	VECTOR vFeet;
//	vec_for_min(vFeet,me); // vFeet.z = distance from player origin to lowest vertex
//	my.z -= vFeet.z;
	SnapIt();
	
	my.ambient = 100;
	set(me,SHADOW | FLAG2);		// cast a shadow and in player mode do not c_trace
	
//	my.emask |= ENABLE_TOUCH | ENABLE_RELEASE;	// e.g. touch->enlight, onclick=select->ent_morph activetilesquare
//	my.event = PlayerMouseEvent;	
}

function attach_sword()
{
       my.ENTITY_TYPE = 100;
       set(my,PASSABLE); // the sword shouldn't slow down the player
       my.ambient = 100;
       while (you != NULL)
       {
               // get the vertex at the bottom of the palm in Med		
               vec_for_vertex (my.skill1, you, 28);
               // get the vertex that separates the thumb and the pointer finger in Med	
               vec_for_vertex (my.skill4, you, 9);
               // compute the vector that will be used by the sword
               vec_diff (my.skill7, my.skill4, my.skill1);
               // rotate the sword accordingly
               vec_to_angle (my.pan, my.skill7);
               // put the origin of the sword in the vertex that is placed at the bottom of the palm
               vec_set (my.x, my.skill1);
               wait (1);
       }
}  

function attach_shield()
{
       my.ENTITY_TYPE = 100;
       set(my,PASSABLE); // the sword shouldn't slow down the player
       my.ambient = 100;
       while (you != NULL)
       {
               // get the vertex at the bottom of the palm in Med		// 28
               vec_for_vertex (my.skill1, you, 467);
               // get the vertex that separates the thumb and the pointer finger in Med		// 9
               vec_for_vertex (my.skill4, you, 446);
               // compute the vector that will be used by the sword
               vec_diff (my.skill7, my.skill4, my.skill1);
               // rotate the sword accordingly
               vec_to_angle (my.pan, my.skill7);
               // put the origin of the sword in the vertex that is placed at the bottom of the palm
               vec_set (my.x, my.skill1);
               wait (1);
       }
}  

action CreatePlayer()
{
	AlignPlayer();
	
	ent_create ("sword1.mdl", my.x, attach_sword);
	ent_create ("shield1.mdl", my.x, attach_shield);
	
	VECTOR vFeet;
	VECTOR vtemp;
	vec_for_min(vFeet,me); 
	while(1)
	{		
		if (Tiles[sx][sy]%HEIGHT_OFFSET==0) { my.z = TilesMid[sx][sy] - vFeet.z ; }
		else if (Tiles[sx][sy]%HEIGHT_OFFSET>=MODELS) { my.z = TilesMin[sx][sy] - vFeet.z + TilesZ[sx][sy]*TileSize; }
		else { my.z = TilesMax[sx][sy] - vFeet.z + RoadLevel; }
		
		// standing
		my.ANIMATION += 4*time_step;
		my.ANIMATION %= 100;
		ent_animate(me,"idle",my.ANIMATION,ANM_CYCLE);
		// turn towards aim - needed by pathastar_turn.c because it determines a good initial direction when strarts moving to destination
		vec_set(vtemp,destobject.x); 
		vec_sub(vtemp,my.x);
		vec_to_angle(my.pan,vtemp); 
		my.tilt=0;
		
		wait(1);
	}
}

// set player place to path start position
function Player_Init()
{
	var tempa = TileToCoord(sx,GridX);
	var tempb = TileToCoord(sy,GridY);
	ent_create( playermodel, vector( tempa , tempb , 0 ), CreatePlayer);	
}

// end point entity
function AlignDestFlag()
{
	destobject = me;
	my.XTILE_NUMBER = fx;
	my.YTILE_NUMBER = fy;
	my.AREA_SIZE = 1;	
	my.ENTITY_TYPE = 101;
	// scale to actual tilesize
	my.scale_x = TileSize/GRIDSIZE;
	my.scale_y = TileSize/GRIDSIZE;
	my.scale_z = TileSize/GRIDSIZE;
//	// set z
//	VECTOR vFeet;
//	vec_for_min(vFeet,me); // vFeet.z = distance from player origin to lowest vertex
//	my.z -= vFeet.z;
	SnapIt();
	
	my.ambient = 100;
	set(me,SHADOW | FLAG2);		// cast a shadow and in player mode do not c_trace
}

action CreateDestFlag()
{
	AlignDestFlag();
	
	VECTOR vFeet;
	vec_for_min(vFeet,me); 
	while(1)
	{				
		if (Tiles[fx][fy]%HEIGHT_OFFSET==0) { my.z = TilesMid[fx][fy] - vFeet.z ; }
		else if (Tiles[fx][fy]%HEIGHT_OFFSET>=MODELS) { my.z = TilesMin[fx][fy] - vFeet.z + TilesZ[fx][fy]*TileSize; }
		else { my.z = TilesMax[fx][fy] - vFeet.z + RoadLevel; }
		wait(1);
	}
}

// set flag place to path end position
function Destination_Init()
{
	var tempa = TileToCoord(fx,GridX);
	var tempb = TileToCoord(fy,GridY);
	ent_create( destmodel, vector( tempa , tempb , 0 ), CreateDestFlag);	
}



