
// tile system: X*Y tiles (defined in main script), 64x64 quant per tile (or 32 but it needs scaling)
var TileSize = GRIDSIZE;
var GridX = SMALLMAPX; 
var GridY = SMALLMAPY; 

// 2 dimensional tile array
var Tiles[SMALLMAPX][SMALLMAPY];  

// camera_control
var ZoomSpeed = 15;
var RotateSpeed = 7.5;
var MoveSpeed = 100;
var ZoomDownLimit = 225;
var ZoomUpLimit = 2500;
var XMax =  SMALLMAPX*GRIDSIZE/2;
var XMin = -SMALLMAPX*GRIDSIZE/2;
var YMax =  SMALLMAPY*GRIDSIZE/2;
var YMin = -SMALLMAPY*GRIDSIZE/2;

// camera data
STRING* CameraModel = "camera_cube.mdl";  // any simple modell
ENTITY* CameraObject;							// that is followed by the camera
var CamZoom = 300;								// camera height 1000 ok
var CamAngle = 270-45; 							// rotation of camera around camera object: 270 = watch from South i.e. watch to North
var CamHeight = 200;								// height of camera object - should be constant but over surface, or constantly over surface

// mouse data
BMAP* mousearrow = "mousearrow.tga";

// active tile data
ENTITY* ActiveTileSquare;												// the square entity
STRING* square1 = "square1.tga";      								// build square
STRING* square2 = "square2.tga";										// endpoint square 
STRING* square3 = "square3.tga";										// wait square 
var square_coord_x , square_coord_y , square_coord_z = 0;	// square entity coordinates
var square_x , square_y = 0;											// square entity grid x,y position
var square_value = 0;													// local value of dataarray
var draw_size = 1;														// some object can be larger than 1 tile, e.g. the tower is 3=3x3

// tile objects 
var water_anim = 1; 										// global variable to synchronize water animation sprites to keep tileable placement
var BuildMode = 0;										// defines what to do
STRING* watertile = "water3+7.tga";					// water sprite - with 7 frame
STRING* dirtroadtile = "earthtile.tga";			// dirt road sprite
STRING* stoneroadtile = "stoneroad.tga";			// stone road sprite
//STRING* woodstile = "grass.tga";					// grass for woods sprite
STRING* wallmodel = "wall01.mdl";					// 1 tile obstacle - because of shadow, mdl is needed - wmb has no shadow
STRING* towermodel = "tower00.mdl";					// 3x3 tile sized obstacle
STRING* treemodel1 = "tree003dds.mdl";				// 1 tile obstacle
STRING* treeshadowmodel1 = "tree003tga_b.mdl";	// 1 tile obstacle used only for casting tree shadow
STRING* treemodel2 = "tree002dds.mdl";				// 1 tile obstacle
STRING* treeshadowmodel2 = "tree002tga_b.mdl";	// 1 tile obstacle used only for casting tree shadow

// path- and ActiveTileSquare-sprite height offset values
var RoadLevel = 0.2;
var PathLevel = 0.5;
var SquareLevel = 1;

// terrain
ENTITY* terrain_entity;

// player and flag models
STRING* playermodel = "pguard3.mdl";
//STRING* destsprite = "flag2.tga";
STRING* destmodel = "flag.mdl";
ENTITY* destobject;

// obstacle seeding from bitmap
BMAP* obstacle_map = "seed_map01.bmp";	// obstacle seeding map
var temp_tx, temp_ty;						// global temporary variables showing tile position - used in actions and functions 

// common in different pathfinding algorithms
var DataArray[SMALLMAPX][SMALLMAPY];  	// to convert tile values to accessibility and move cost values
var sx = 40;									// "start" tile position
var sy = 40;
var fx = 80;									// "finish" tile position 
var fy = 80;
var tx, ty; 	      						// temporary tile position
var found;					   				// found: destination is reached
var maxpath;									// to maximize cycle time and path length, to avoid infinite cycles and too long calculations
var pathlength = 0;							// show actual path length
// the resulting path array
var pathX[MAXPATHLENGTH], pathY[MAXPATHLENGTH]; 			
// they hold x;y coordinate values of path steps
var pathDir[MAXPATHLENGTH];
var cycle_time = 0;							// to display cycle number in the end

// pathfinder user settings
var drawalltiles = 1;						// what to display after path search, 1: draw all the possible paths , 0: draw only the path dots
var path_type = 1;							// 1 direction map, 2 A* manhattan, 3 A* euclidian, 4 A* diagonal shortcut 
var heur_value = 10;
var total_move_cost = 0;
var depthfirst = 0;

// path search time needed
float time_1, time_2;
float time_elapsed;
STRING* timer_value = "0.000";

// representing the paths
ENTITY* ArrowArray[SMALLMAPSIZE];
ENTITY* PathArray[MAXPATHLENGTH];
STRING* path_dot = "path_dot.tga";	// pathfinder sprite

//////////////// displaying information: TEXT and PANEL definitions
TEXT* info_txt =	
{
  layer = 1;
  pos_x = 512;
  pos_y = 720;
  strings = 1;
  font = "Arial#16b";
  string ("Move/rotate camera by moving mouse to screen edge", "Zoom by mouse wheel" , "Press [SPACE] to clear map" );
  flags = CENTER_X | OUTLINE | SHOW;
} 

TEXT* timer_txt =	
{
  layer = 1;
  pos_x = 512;
  pos_y = 650;
  strings = 1;
  font = "Arial#36b";
  string ("Last search time interval [sec]: " , timer_value);
  flags = CENTER_X | OUTLINE | SHOW;
} 

TEXT* wait_txt =	
{
  layer = 1;
  pos_x = 512;
  pos_y = 600;
  blue=0;
  green=0;
  red=255;
  strings = 1;
  font = "Arial#36b";
  string ("Please wait...");
  flags = CENTER_X | OUTLINE;
} 

PANEL* grid_position =
{
	flags = TRANSLUCENT | SHADOW | SHOW;
	alpha = 75;	
	pos_x = 850; 
	pos_y = 0;
	bmap = "position.bmp";
	layer = 10;	
	
	digits(15, 10, "Coordinates:", "Arial#14b", 0, 0);
	digits(15, 25, "x = %0.f", "Arial#14b", 1, square_coord_x);
	digits(15, 40, "y = %0.f", "Arial#14b", 1, square_coord_y);
	digits(15, 55, "z = %0.f", "Arial#14b", 1, square_coord_z);
	
	digits(15, 70, "Tile position:", "Arial#14b", 0, 0);
	digits(15, 85, "x = %0.f", "Arial#14b", 1, square_x);
	digits(15, 100, "y = %0.f", "Arial#14b", 1, square_y);
	digits(15, 115, "Tiles = %0.f", "Arial#14b", 1, square_value);
	
	digits(15, 140, "Build mode: %0.f", "Arial#14b", 1, BuildMode);
	digits(15, 155, "Path draw mode: %0.f", "Arial#14b" , 1, drawalltiles);
	digits(15, 170, "Obstacle size: %0.f", "Arial#14b" , 1, draw_size);
	
	digits(15, 185, "Path type: %0.f", "Arial#14b", 1, path_type);
	digits(15, 200, "Last cycle num: %0.f", "Arial#14b", 1, cycle_time);
	digits(15, 215, "Last path length: %0.f", "Arial#14b", 1, pathlength);
	digits(15, 230, "Last move cost: %0.f", "Arial#14b", 1, total_move_cost);
	
}

PANEL* work_in_progress =
{
	pos_x = 924; 
	pos_y = 300;
	bmap = "work.bmp";	
	layer = 1;
}

PANEL* path_error =
{
	pos_x = 924; 
	pos_y = 300;
	bmap = "error.bmp";	
	layer = 3;
}

PANEL* path_found =
{
	flags = SHADOW;
	pos_x = 924; 
	pos_y = 300;
	bmap = "found.bmp";	
	layer = 2;
}

//////////////////////////////////////////////
// iso sys function pre-declarations

// initialization
function Interface_Init();

// camera
action SetCamera();   
function TestLimits();
function Camera_Init();

// tile system
function TileSys_Init();

// mouse camera operations
function Mouse_Init();

// ActiveTilePos functions
function Mouse3D();
function Mouse3D2();
function SnapIt();
function ShowPosition();	// important unit conversions
function Square_Init();
function Square();

// level detection
function PosInfo3D(tempx, tempy);
function PosInfo3D2(tempx, tempy);

// unit conversion
function TileToCoord( xytile , grid );

// player settings
action AlignPlayer();
function MovePlayer();
function Player_Init();

// destination flag settings
action AlignDestFlag();
function Destination_Init();

// object removal - can be unified
function RemoveEntity(tempx,tempy);
function RemoveEntity2(tempx,tempy);

// tile value handling
function TileFill(num);
function AreaFill(num);

// object sprite and model settings
action AlignTile();
action AlignModel();
action AlignModel2();

// object placing/removing
function PlaceObject();
function PlaceObject2();

// mode changes
function ChangeMode();

// path related common functions
function EndOfPathfinding();
function Path_Cost_Calculation();

// user interface
function call_selected_pathfinder();
function toggle_draw_mode();
function toggle_depthfirst();
function change_algorythm(button_number);
function change_buildmode(button_number);

function Load_Map();
function ClearMap();

//// control panel 1-2
BMAP* pathbtn_on_bmp = "pathbtn_on.bmp";
BMAP* pathbtn_off_bmp = "pathbtn_off.bmp";
BMAP* radiobtn_on_bmp = "radiobtn_on.bmp";
BMAP* radiobtn_off_bmp = "radiobtn_off.bmp";
BMAP* checkbox_on_bmp = "checkbox_on.bmp";
BMAP* checkbox_off_bmp = "checkbox_off.bmp";
BMAP* horiz_slider_bmp = "hslider.bmp";
BMAP* slider_knob_bmp = "hsliderknob.bmp";
BMAP* loadbtn_on_bmp = "loadbtn_on.bmp";
BMAP* loadbtn_off_bmp = "loadbtn_off.bmp";

PANEL* info_panel_1 =
{
	flags = TRANSLUCENT | SHADOW | SHOW | OVERLAY;
	alpha = 75;	
	pos_x = 768; 
	pos_y = 512;
	bmap = "position.bmp";
	layer = 10;	
	
	button_radio (15, 30, radiobtn_on_bmp, radiobtn_off_bmp, radiobtn_off_bmp, change_algorythm, NULL,  NULL);	
	button_radio (15, 50, radiobtn_on_bmp, radiobtn_off_bmp, radiobtn_off_bmp, change_algorythm, NULL, NULL);
	button_radio (15, 70, radiobtn_on_bmp, radiobtn_off_bmp, radiobtn_off_bmp, change_algorythm, NULL, NULL);
	button_radio (15, 90, radiobtn_on_bmp, radiobtn_off_bmp, radiobtn_off_bmp, change_algorythm, NULL, NULL);
	
	button (15, 215, pathbtn_on_bmp, pathbtn_off_bmp, pathbtn_on_bmp, call_selected_pathfinder , NULL, NULL);
	button_toggle (15, 150, checkbox_on_bmp, checkbox_off_bmp, checkbox_on_bmp, checkbox_off_bmp, toggle_draw_mode , NULL, NULL);
	button (130, 215, loadbtn_on_bmp, loadbtn_off_bmp, loadbtn_on_bmp, Load_Map , NULL, NULL);
		
	slider (15, 180, horiz_slider_bmp, slider_knob_bmp, 0, 20, heur_value);
	
	digits(15, 5, "Select a pathfinding algorythm", "Arial#16b", 0, 0);
	digits(35, 30, "[F1] Direction Map (ext. Breadth-First)", "Arial#14b", 0, 0);
	digits(35, 50, "[F2] A* with heuristic: Manhattan", "Arial#14b", 0, 0);
	digits(35, 70, "[F3] A* with heuristic: Euclidian", "Arial#14b", 0, 0);
	digits(35, 90, "[F4] A* with heuristic: Diagonal shortcut", "Arial#14b", 0, 0);
	digits(35, 110, "[F5] A* without H: Dijktsra", "Arial#14b", 0, 0);
	digits(35, 130, "[F6] A* withuot G: Depth-First", "Arial#14b", 0, 0);
	digits(35, 150, "[F12] Show only final path (draws faster)", "Arial#14b", 0, 0);
	digits(125, 177.5, "Heuristic Weight: %0.2f", "Arial#14b", 1, heur_value);
	digits(15, 195, "if this value is 0, A* works as Dijktsra", "Arial#14b", 0, 0);
	
	button_radio (15, 110, radiobtn_on_bmp, radiobtn_off_bmp, radiobtn_off_bmp, change_algorythm, NULL, NULL);
	button_toggle (15, 130, checkbox_on_bmp, checkbox_off_bmp, checkbox_on_bmp, checkbox_off_bmp, toggle_depthfirst , NULL, NULL);
}

PANEL* info_panel_2 =
{
	flags = TRANSLUCENT | SHADOW | SHOW | OVERLAY;
	alpha = 75;	
	pos_x = 0; 
	pos_y = 512;
	bmap = "position.bmp";
	layer = 10;	
		
	button_radio (15, 50, radiobtn_on_bmp, radiobtn_off_bmp, radiobtn_off_bmp, change_buildmode, NULL, NULL);
	button_radio (15, 70, radiobtn_on_bmp, radiobtn_off_bmp, radiobtn_off_bmp, change_buildmode, NULL, NULL);
	button_radio (15, 90, radiobtn_on_bmp, radiobtn_off_bmp, radiobtn_off_bmp, change_buildmode, NULL, NULL);
	button_radio (15, 110, radiobtn_on_bmp, radiobtn_off_bmp, radiobtn_off_bmp, change_buildmode, NULL, NULL);
	button_radio (15, 130, radiobtn_on_bmp, radiobtn_off_bmp, radiobtn_off_bmp, change_buildmode, NULL, NULL);
	button_radio (15, 150, radiobtn_on_bmp, radiobtn_off_bmp, radiobtn_off_bmp, change_buildmode, NULL, NULL);
		
	button_radio (15, 200, radiobtn_on_bmp, radiobtn_off_bmp, radiobtn_off_bmp, change_buildmode, NULL, NULL);
	
	digits(15, 10, "Place obstacles:", "Arial#16b", 0, 0);
	digits(15, 30, "(left click - build, right click - delete)", "Arial#14b", 0, 0);
	digits(35, 50, "[1] water - no access", "Arial#14b", 0, 0);
	digits(35, 70, "[2] stone road - fastest move", "Arial#14b", 0, 0);
	digits(35, 90, "[3] dirt road - faster move", "Arial#14b", 0, 0);
	digits(35, 110, "[4] wall - no access", "Arial#14b", 0, 0);
	digits(35, 130, "[5] tower - no access", "Arial#14b", 0, 0);
	digits(35, 150, "[6] tree - no access & slow move area", "Arial#14b", 0, 0);
	
	digits(15, 180, "Modify path:", "Arial#16b", 0, 0);
	digits(40, 200, "[0] replace start and endpoint", "Arial#14b", 0, 0);
	digits(15, 220, "(left click - start, right click - destination)", "Arial#14b", 0, 0);
}

////////////////////////////////////
// it calls pathfinding
#include "pathdirmap.c"
#include "pathastar.c"
////////////////////////////////////////////////
// start of the system
function Interface_Init()
{			
	// necessary function calls 
	TileSys_Init();		// tile value nulling
	wait(5);
	Camera_Init(); 		// CameraObject entity creation , to be followed by camera
	wait(2);
	Mouse_Init();			// mouse start
	Square_Init();   		// ActiveTileSquare creation and settings
	
	Player_Init();			// create player model
	Destination_Init();	// create destination flag model
	ChangeMode();			// set the proper skin of ActiveTileSquare 
		
	// panels settings
	button_state(info_panel_1,1,ON); 
	button_state(info_panel_2,7,ON); 
		
	// temporary function calls
	wait(5);
	on_space = ClearMap;  // clear obstacles from map and delete tile values

	// object placement
	on_mouse_left = PlaceObject;
	// object moving/deleting
	on_mouse_right = PlaceObject2;	
	
	while(1)
	{
		// global variable sets the water sprite animation speed
		water_anim += 0.25 * time_step;
	   water_anim %= 7;
		
		if (key_any)
		  if (BuildMode!=255)		// path showing state
		  {
			// build / delete mode
			if(key_1) {BuildMode = 1;}		// water placement - sprite creation
			if(key_2) {BuildMode = 2;}		// stone road placement - sprite creation
			if(key_3) {BuildMode = 3;}		// dirt road placement - sprite creation
			if(key_4) {BuildMode = 4;}		// wall placement - 1-tile model creation
			if(key_5) {BuildMode = 5;}		// tower placement - multi-tile model creation
			if(key_6) {BuildMode = 6;}		// tree placement - 1-tile model creation
				
			// change path endpoints mode
			if(key_0) {BuildMode = 0;}		// replace path endpoints mode
			
			// update buttons
			if (BuildMode>0) button_state( info_panel_2, BuildMode , ON );
			else button_state( info_panel_2, 7 , ON );
			
			// toggle path representation mode
			if (key_f12)
			{
				while (key_f12) {wait (1);}
				
				drawalltiles = 1 - drawalltiles;  // 0 or 1
				// update button
				button_state( info_panel_1,6, (1-drawalltiles) );
			}		
			if (key_f6)
			{
				while (key_f6) {wait (1);}
				
				depthfirst = 1 - depthfirst;  // 0 or 1
				// update button
				button_state( info_panel_1,9, (depthfirst) );
			}
			
			// start path calculations
			if(key_f1) 
				{
					set(wait_txt,SHOW);
					// update panel menu values
					path_type = 1;
					button_state(info_panel_1,1,ON); 
					change_algorythm(1); // call pathfinder with default heuristic value
					BuildMode = 255;
					PathFind00();
				}
			else if(key_f2) 
				{
					set(wait_txt,SHOW);
					// update panel menu values
					path_type = 2;
					button_state(info_panel_1,2,ON); 
					change_algorythm(2); // call pathfinder with default heuristic value
					BuildMode = 255;
					PathFind01(1);
				}
			else if(key_f3) 
				{
					set(wait_txt,SHOW);
					// update panel menu values
					path_type = 3;
					button_state(info_panel_1,3,ON);
					change_algorythm(3); // call pathfinder with default heuristic value
					BuildMode = 255;
					PathFind01(2);
				}
			else if(key_f4) 
				{
					set(wait_txt,SHOW);
					// update panel menu values
					path_type = 4;
					button_state(info_panel_1,4,ON);
					change_algorythm(4); // call pathfinder with default heuristic value
					BuildMode = 255;
					PathFind01(3);
				}
			else if(key_f5) 
				{
					set(wait_txt,SHOW);
					// update panel menu values
					path_type = 0;
					button_state(info_panel_1,8,ON);
					change_algorythm(8); // call pathfinder with default heuristic value
					BuildMode = 255;
					PathFind01(0);
				}
				
			ChangeMode();			// change ActiveTileSquare's bitmap due to BuildMode		
			}		
		wait(1);
	}	
}

/////////////////////////////////////////////////
// camera handling functions

// make it undetectable by c_trace calls
action SetCamera()
{
	CameraObject = me;
	my.ENTITY_TYPE = 103;
//	my.z = CamHeight;
	set( me , PASSABLE | INVISIBLE | FLAG2 );
}

// set min and max coordinates of camera object
function TestLimits()
{
	CameraObject.x = maxv(minv(CameraObject.x, XMax), XMin);
	CameraObject.y = maxv(minv(CameraObject.y, YMax), YMin);
}

// place camera object entity to be followed by camera
function Camera_Init()
{  	
	ent_create( CameraModel, vector(-1600,-1600,CamHeight), SetCamera);
	
	while(1)
	{                
		camera.z = CamZoom;
		camera.x = CameraObject.x + CamZoom * cos(CamAngle); 
		camera.y = CameraObject.y + CamZoom * sin(CamAngle); 
		
		var temp;
		
		vec_set(temp,CameraObject.x);
		vec_sub(temp,camera.x);
		vec_to_angle(camera.pan,temp);              
		CamAngle %= 360;           
		
		wait(1);
	}
}

//////////////////////////////////////////////////
// tile value nulling
function TileSys_Init()
{
	var i,j;
	for (j=0 ; j<GridX ; j++)
		for (i=0 ; i<GridY ; i++)
			Tiles[j][i]=0;
}

///////////////////////////////////
// mouse related functions

// mouse initialization and camera movements
function Mouse_Init()
{
	mouse_map = mousearrow;
	mouse_mode = 1;
	while (mouse_mode > 0)
	{
		vec_set(mouse_pos , mouse_cursor);                       
		
		// camera movements by mouse
		// forward
		if (mouse_pos.y <= 2)
		{
			CameraObject.y += sin(camera.pan) * MoveSpeed * time_step + (key_shift * sin(camera.pan) * MoveSpeed * time_step);
			CameraObject.x += cos(camera.pan) * MoveSpeed * time_step + (key_shift * cos(camera.pan) * MoveSpeed * time_step);
			TestLimits();
		}
		// back
		if (mouse_pos.y >= screen_size.y -2)
		{
			CameraObject.y -= sin(camera.pan) * MoveSpeed * time_step + (key_shift * sin(camera.pan) * MoveSpeed * time_step);
			CameraObject.x -= cos(camera.pan) * MoveSpeed * time_step + (key_shift * cos(camera.pan) * MoveSpeed * time_step);
			TestLimits();
		}
		// left
		if ( (mouse_pos.x <= 2) && (mouse_pos.y > screen_size.y/2))
		{
			CameraObject.y += cos(camera.pan) * MoveSpeed * time_step + (key_shift * cos(camera.pan) * MoveSpeed * time_step);
			CameraObject.x -= sin(camera.pan) * MoveSpeed * time_step + (key_shift * sin(camera.pan) * MoveSpeed * time_step);
			TestLimits();
		}
		// right
		if ((mouse_pos.x >= screen_size.x -2) && (mouse_pos.y > screen_size.y/2))
		{
			CameraObject.y -= cos(camera.pan) * MoveSpeed * time_step + (key_shift * cos(camera.pan) * MoveSpeed * time_step);
			CameraObject.x += sin(camera.pan) * MoveSpeed * time_step + (key_shift * sin(camera.pan) * MoveSpeed * time_step);
			TestLimits();
		}
		// rotate left
		if ( (mouse_pos.x <= 2) && (mouse_pos.y <= screen_size.y/2))
		{
			CamAngle += RotateSpeed * time_step;                        
		}
		// rotate right
		if ((mouse_pos.x >= screen_size.x -2) && (mouse_pos.y <= screen_size.y/2))
		{
			CamAngle -= RotateSpeed * time_step;                      
		}
		// zoom
		if (!(mickey.z==0)) 
		{
			CamZoom += ZoomSpeed * 0.05 * mickey.z ;
			if (CamZoom < ZoomDownLimit)
			{
				CamZoom = ZoomDownLimit;
			}	
			if (CamZoom > ZoomUpLimit)
			{
				CamZoom = ZoomUpLimit;
			}
		}
		
		wait(1);
	}
}

//////////////////////////////////
// ActiveTileSquare related functions

// get mouse position on terrain surface
function Mouse3D()
{
	VECTOR pos1, pos2;

	vec_set( pos1 , vector(mouse_pos.x,mouse_pos.y,0) );
	vec_for_screen (pos1, camera);
	
	vec_set( pos2 , vector(mouse_pos.x,mouse_pos.y,20000) ); // should be a big value       		
	vec_for_screen (pos2, camera);

	c_trace (pos1.x, pos2.x, IGNORE_SPRITES | IGNORE_PASSABLE | IGNORE_MODELS | IGNORE_MAPS ); 
	// if there is a HIT_TARGET event, now "target" and "hit.x" hold the coordinates of the hit point 
	// ignoring tile sprites, models, map-entities and other passable entities including passable terrains (water)      
}

// get mouse position on terrain and some special model/map-entity surfaces
function Mouse3D2()
{
	VECTOR pos1, pos2;

	vec_set( pos1 , vector(mouse_pos.x,mouse_pos.y,0) );
	vec_for_screen (pos1, camera);
	
	vec_set( pos2 , vector(mouse_pos.x,mouse_pos.y,20000) ); // should be a big value       		
	vec_for_screen (pos2, camera);

	c_trace (pos1.x, pos2.x, IGNORE_SPRITES | IGNORE_FLAG2); 
	// if there is a HIT_TARGET event, now "target" and "hit.x" hold the coordinates of the hit point 
	// ignoring tile sprites, and flag2 entities (those have no walkable surface - for further development)      
}


// snapping to tile midpoint, within grid limits
function SnapIt()
{  
	// x-axis snapping
	if (my.x < 0)
	{
		// snap
		my.x = TileSize * (integer(my.x / TileSize)) - (TileSize / 2);		
		// limit   
		if (my.x < -((GridX / 2) * TileSize - (TileSize / 2)))
		{
			my.x = -((GridX / 2) * TileSize - (TileSize / 2));
		}
	}
	else if (my.x >= 0)
	{
		// snap
		my.x = TileSize * (integer(my.x / TileSize)) + (TileSize / 2);
		// limit
		if (my.x > ((GridX / 2) * TileSize - (TileSize / 2)))
		{
			my.x = ((GridX / 2) * TileSize - (TileSize / 2));
		}
	}       
	
	// y-axis snapping
	if (my.y < 0)
	{
		// snap
		my.y = TileSize * (integer(my.y / TileSize)) - (TileSize / 2);
		// limit
		if (my.y < -((GridY / 2) * TileSize - (TileSize / 2)))
		{
			my.y = -((GridY / 2) * TileSize - (TileSize / 2));
		}
	}
	else if (my.y >= 0)
	{
		// snap
		my.y = TileSize * (integer(my.y / TileSize)) + (TileSize / 2);
		// limit
		if (my.y > ((GridY / 2) * TileSize - (TileSize / 2)))
		{
			my.y = ((GridY / 2) * TileSize - (TileSize / 2));
		}
	}  
}

// determine ActiveTileSquare x,y tile position and value, for further calculations and display
function ShowPosition()
{  	
	square_coord_x = ActiveTileSquare.x;
	square_coord_y = ActiveTileSquare.y;
	square_coord_z = ActiveTileSquare.z;
	
	square_x = integer ( (ActiveTileSquare.x + 1 * GridX / 2 * TileSize)/TileSize );
	square_y = integer ( (ActiveTileSquare.y + 1 * GridY / 2 * TileSize)/TileSize );

	//to display the value stored in the actual node
	square_value = Tiles[square_x][square_y];
}

// set ActiveTileSquare place and properties, and refresh in each frame
function Square()
{          
	// give an entity name to square sprite eablig tracking x,y coordinates
	ActiveTileSquare = me;
	
	my.ENTITY_TYPE = 102;
	set( me , PASSABLE );			// passable to avoid c_tracing, | ZNEAR render over other entities
	my.scale_x = TileSize/GRIDSIZE;
	my.scale_y = TileSize/GRIDSIZE;
	// horizintal alignment
	vec_set( my.pan , vector(270,90,0) ); 

	while(1)
	{         	                
		if (BuildMode == 0) Mouse3D2();	// call c_tracing to get 3D position on any surface  								
		if (BuildMode != 0) Mouse3D(); 	// call c_tracing to get 3D position on terrain	
		
		// move square only if mouse points within terrain - else it remains in previous position		
		if (HIT_TARGET) 					// without this check it would point to Node (0,0) if mouse points out of terrain
		{
			vec_set( my.x, target);    // or hit.x
			my.z += SquareLevel;       // a bit over surface
			SnapIt();  						// snap to middle of tile
			ShowPosition();
			
			if (BuildMode == 0) 
			{
				PosInfo3D2(my.x,my.y); 
				my.z = hit.z + SquareLevel;
			}
		}		
		
		wait(1);
	}
}

// create ActiveTileSquare anywhere
function Square_Init()
{
	ent_create( square1, nullvector, Square);	
}

/////////////////////////////////////////
// different terrain level detection and coordinate conversion functions

// using c-trace for getting tile terrain surface information at any x;y coordinate position
function PosInfo3D(tempx, tempy)
{
	VECTOR pos1, pos2;

	vec_set( pos1 , vector(tempx, tempy, -100) );
	vec_set( pos2 , vector(tempx, tempy, 1000) ); // should be a big value       		
	
	c_trace (pos2, pos1, IGNORE_SPRITES | IGNORE_MODELS | IGNORE_MAPS );    
}

// using c-trace for getting tile building surface information at any x;y coordinate position
function PosInfo3D2(tempx, tempy)
{
	VECTOR pos1, pos2;

	vec_set( pos1 , vector(tempx, tempy, -100) );
	vec_set( pos2 , vector(tempx, tempy, 1000) ); // should be a big value       		
	
	c_trace (pos2, pos1, IGNORE_SPRITES | IGNORE_FLAG2 );    
}

// determine x/y coordinate of a tile by reverse calculation from: square_x = integer ( (ActiveTileSquare.x + 1 * GridX / 2 * TileSize)/TileSize );
function TileToCoord( xytile , grid )
{	
	var temp = ( xytile * TileSize ) - 1 * grid / 2 * TileSize +1;	// +1 isneeded for correct snapping
	return( temp );
}

//////////////////////////////////////
// setting entities showing path start and end positions

// start point entity
action AlignPlayer()
{
	my.ANIMATION = 0;
	my.XTILE_NUMBER = sx;
	my.YTILE_NUMBER = sy;
	my.AREA_SIZE = 1;	
	my.ENTITY_TYPE = 100;
	player = me;	
		
	// scale to actual tilesize
	my.scale_x = TileSize/GRIDSIZE;
	my.scale_y = TileSize/GRIDSIZE;
	my.scale_z = TileSize/GRIDSIZE;
	// set z
	VECTOR vFeet;
	vec_for_min(vFeet,me); // vFeet.z = distance from player origin to lowest vertex
	my.z -= vFeet.z;
	SnapIt();
	
	my.ambient = 100;
	set(me,SHADOW | FLAG2);		// cast a shadow and in player mode do not c_trace
	
//	my.emask |= ENABLE_TOUCH | ENABLE_RELEASE;
//	my.event = PlayerMouseEvent;	
}

function MovePlayer()
{
	VECTOR vFeet;
	vec_for_min(vFeet,me); 
	while(1)
	{
		PosInfo3D(my.x, my.y);
		if (HIT_TARGET) 
		{
			my.z = hit.z - vFeet.z; // always place player's feet on the ground		
			
			// standing
			my.ANIMATION += 4*time_step;
			my.ANIMATION %= 100;
			ent_animate(me,"idle",my.ANIMATION,ANM_CYCLE);
		}
		wait(1);
	}
}

// set player place to path start position
function Player_Init()
{
	var tempa = TileToCoord(sx,GridX);
	var tempb = TileToCoord(sy,GridY);
	ent_create( playermodel, vector( tempa , tempb , 0 ), AlignPlayer);	
	// call move loop only once
	me = player;
	MovePlayer();
}

// end point entity
action AlignDestFlag()
{
	destobject = me;
	my.XTILE_NUMBER = fx;
	my.YTILE_NUMBER = fy;
	my.AREA_SIZE = 1;	
	my.ENTITY_TYPE = 101;
	// scale to actual tilesize
	my.scale_x = TileSize/GRIDSIZE;
	my.scale_y = TileSize/GRIDSIZE;
	my.scale_z = TileSize/GRIDSIZE;
	// set z
	VECTOR vFeet;
	vec_for_min(vFeet,me); // vFeet.z = distance from player origin to lowest vertex
	my.z -= vFeet.z;
	SnapIt();
	// check z
	PosInfo3D(my.x, my.y);
	if (HIT_TARGET) my.z = hit.z - vFeet.z; 
	my.ambient = 100;
	set(me,SHADOW | FLAG2);		// cast a shadow and in player mode do not c_trace
}

// set flag place to path end position
function Destination_Init()
{
	var tempa = TileToCoord(fx,GridX);
	var tempb = TileToCoord(fy,GridY);
	ent_create( destmodel, vector( tempa , tempb , 0 ), AlignDestFlag);	
}

/////////////////////////////////////////
// object removal

// remove entity from the tile given
function RemoveEntity(tempx,tempy)
{
	you = ent_next(NULL); // get first entity
 	while (you) // repeat until there are no more entities
	{ 
		// check position stored in entity skills
		if ( (you.XTILE_NUMBER==tempx) && (you.YTILE_NUMBER==tempy) )
			if (you.ENTITY_TYPE < 100)
			{
				ent_remove(you);  			
				break;			// end of loop
			}				
 		you = ent_next(you); // get next entity
	}	
}

// remove entity from the tile given, and detect its draw size enabling to clear all the tile values in the surroundings
function RemoveEntity2(tempx,tempy)
{
	you = ent_next(NULL); // retrieve first entity
 	while (you) // repeat until there are no more entities
	{ 
		if ( (you.XTILE_NUMBER==tempx) && (you.YTILE_NUMBER==tempy) )
			if (you.ENTITY_TYPE < 100)
			{
				draw_size = you.AREA_SIZE;  // used for removing a large-area-entity occupying more tile , those should be cleared all, after this function terminates
				ent_remove(you);  				
				return(1);	// exit and feedback: object deleted		
			}				
 		you = ent_next(you); // get next entity
	}
	return(0);	// feedback: nothing was deleted
}

/////////////////////////////////////////////////
// writing new values into Tiles array

// filling the actual tile
function TileFill(num)
{                                     		
	Tiles[ temp_tx ][ temp_ty ] = num;	
}
 
 // filling more than 1 tiles, more exactly: draw_size x draw_size tiles
function AreaFill(num)
{                                     
	var i, j;
	var tempa, tempb;           

	tempa = temp_tx - integer(draw_size/2) ;
	tempb = temp_ty - integer(draw_size/2) ;
		
	for (i=0 ; i< (draw_size) ; i++)	
		for (j=0 ; j< (draw_size) ; j++)
			{
				// preventing from trying overwriting tiles out of grid boundaries
				if ( (tempa+i>=0)  && (tempa+i<GridX) && (tempb+j>=0) && (tempb+j<GridY) )
				{	
					Tiles[ tempa+i ][ tempb+j ] = num;
					
					// if there is any other entity in the building area - remove it, except the placement tile
//					if ( !((tempa+i == temp_tx) && (tempb+j == temp_ty)) )	// not needed - now it is called always before entity creation
						
					RemoveEntity(tempa+i,tempb+j);
				}
			}
}

////////////////////////////////////////
// obstacle creation functions

// setting a sprite
action AlignTile()
{	
	my.XTILE_NUMBER = temp_tx;
	my.YTILE_NUMBER = temp_ty;
	my.AREA_SIZE = 1;
	my.ENTITY_TYPE = 1;
	
	// horizintal alignment
	vec_set( my.pan , vector(270,90,0) );       
	// a bit over surface but under the mouse following square sprite
	my.z += RoadLevel;
	// scale to actual tilesize
	my.scale_x = 2 * TileSize/GRIDSIZE ;
	my.scale_y = 2 * TileSize/GRIDSIZE ;
	// the same snap function  
	SnapIt();
	// check z
	PosInfo3D(my.x, my.y);
	if (HIT_TARGET) my.z = hit.z + RoadLevel; 
	my.ambient = 100;
	set( me , PASSABLE | SHADOW);		// to avoid collision
	
	// water animation - all is synchronized by water_anim global variable to be tileable making rivers
	if (BuildMode == 1)
	{
		// animating water flow
		while(1)
		{
	      my.frame = water_anim;  
			wait(1);
		}
	}
}

// making the touched model transparent - easier obstacle management
function MouseEvent()
{
	switch (event_type)
  	{
  		case EVENT_TOUCH:
  			set(my, TRANSLUCENT );  //   
			my.alpha = 33;
			return;
		case EVENT_RELEASE:
			reset(my, TRANSLUCENT );  //   
			my.alpha = 100;
			return;
	}	
}

// setting a model
action AlignModel()
{	
	my.XTILE_NUMBER = temp_tx;
	my.YTILE_NUMBER = temp_ty;
	my.AREA_SIZE = draw_size;	
	my.ENTITY_TYPE = 2;
	
	// scale to actual tilesize
	my.scale_x = TileSize/GRIDSIZE;
	my.scale_y = TileSize/GRIDSIZE;
	my.scale_z = TileSize/GRIDSIZE;
	
	c_setminmax(me);
//	my.z += -my.min_z -5;
	VECTOR vFeet;
	vec_for_min(vFeet,me); // vFeet.z = distance from player origin to lowest vertex
	my.z -= vFeet.z +5;	
	SnapIt();
	// check z
	PosInfo3D(my.x, my.y);
	if (HIT_TARGET) my.z = hit.z - vFeet.z; 
	
	set( me , POLYGON);
	if (BuildMode != 6) set( me , SHADOW );
	if (BuildMode == 6) set( me , FLAG2 );		// to exclude vegetation from some c_tracing that determines both terrain and building top surface
	
	my.ambient = 100;
	my.emask |= ENABLE_TOUCH | ENABLE_RELEASE;
	my.event = MouseEvent;
}

// setting a shadow maker model of another model having transparent skin thus casting no shadow
action AlignModel2()
{	
	my.XTILE_NUMBER = temp_tx;
	my.YTILE_NUMBER = temp_ty;
	my.AREA_SIZE = draw_size;	
	my.ENTITY_TYPE = 3;
	
	// scale to actual tilesize
	my.scale_x = TileSize/GRIDSIZE;
	my.scale_y = TileSize/GRIDSIZE;
	my.scale_z = TileSize/GRIDSIZE;
	
//	c_setminmax(me);
//	my.z += -my.min_z -5;
	VECTOR vFeet;
	vec_for_min(vFeet,me); // vFeet.z = distance from player origin to lowest vertex
	my.z -= vFeet.z +5;	
	SnapIt();
	// check z
	PosInfo3D(my.x, my.y);
	if (HIT_TARGET) my.z = hit.z - vFeet.z; 
	
	my.ambient = 100;
	set( me , SHADOW | INVISIBLE | FLAG2 );		// used for casting a shadow of a tree model having transparent skin, and also to avoid some c_tracings
	
	my.emask |= ENABLE_TOUCH | ENABLE_RELEASE;
	my.event = MouseEvent;
}

///////////////////////////////////////////////////
// user mouse operations

// left click operations
function PlaceObject()
{	
	STRING* temp_str;
	var rnd;
	
	// if  mouse is over a panel then exit this function
	if ( (mouse_pos.x<256) && (mouse_pos.y>512) ) {return;}	// control panel 2
	if ( (mouse_pos.x>768) && (mouse_pos.y>512) ) {return;}	// control panel 1
	if ( (mouse_pos.x>850) && (mouse_pos.y<256) ) {return;}	// info panel
	
	// replace player model
	if (BuildMode == 0)
		if ( (square_x!=fx) || (square_y!=fy) )
			if ( (Tiles[ square_x ][ square_y ]<4) && (Tiles[ square_x ][ square_y ]!=1) ) // tile is neither building, nor water, nor tree
			{
				// change start tile
				sx = square_x;
				sy = square_y;
				// replace 
				var tempa = TileToCoord(sx,GridX);
				var tempb = TileToCoord(sy,GridY);
				me = player;
				my.x = tempa;
				my.y = tempb;
				my.pan = camera.pan;
				AlignPlayer();		
			}
	
	// build
	if ((BuildMode > 0))					// && (BuildMode < 255)
	{
		if ( Tiles[ square_x ][ square_y ]==0 )		// if tile is empty
		{
			// global temporary variables used in "Align-Tile/Model/Model2" actions and "Tile/Area-Fill" functions
			temp_tx = square_x;
			temp_ty = square_y;
			
			Mouse3D(); // detect again mouse position on map
			if (HIT_TARGET) // do drawing only if mouse points to terrain - else target has a value of 0,0 which would result in drawing at Node (0,0)
			{					
				// sprite creation - any free tile							
				// water - sprite creation
				if (BuildMode == 1) 
				{
					TileFill(1);					
					ent_create(watertile, target, AlignTile);
				}				
				// road - sprite creation
				else if (BuildMode == 2) 
				{
					TileFill(2);					
					ent_create(stoneroadtile, target, AlignTile);
				}					
				else if (BuildMode == 3) 
				{
					TileFill(3);
					ent_create(dirtroadtile, target, AlignTile);					
				}	
				
				// model creation - free tile if neither start nor end of path
				if ( !((temp_tx==sx) && (temp_ty==sy)) && !((temp_tx==fx)  && (temp_ty==fy)) )		
				{				
					// wall
					if (BuildMode == 4) 
					{
						draw_size = 1;		// its draw area size in tiles
						TileFill(1);
						ent_create(wallmodel, target, AlignModel);					
					}				
					// tower
					else if (BuildMode == 5) 
					{
						draw_size = 3;		// its draw area size in tiles
						AreaFill(1);
						ent_create(towermodel, target, AlignModel);															
					}				
					// tree
					else if (BuildMode == 6) 
					{
						draw_size = 5;		// its draw area size in tiles
						AreaFill(4);		// around the tree thearea is woods
						TileFill(1);		// the tree itselfis an obstacle	
						
						random_seed(0); 	// starts with a really random initial value
						rnd = random(2);
						
						if (rnd<1) temp_str = treemodel1;
						else temp_str = treemodel2;	
						ent_create(temp_str, target, AlignModel);	// the visible model
						
						if (rnd<1) temp_str = treeshadowmodel1;
						else temp_str = treeshadowmodel2;						
						ent_create(temp_str, target, AlignModel2);	// an invisible model for proper shadow					
					}
				}
			}
		}		     
	}    

}

// right click operations
function PlaceObject2()
{
	// if  mouse is over a panel then exit this function
	if ( (mouse_pos.x<256) && (mouse_pos.y>512) ) {return;}	// control panel 2
	if ( (mouse_pos.x>768) && (mouse_pos.y>512) ) {return;}	// control panel 1
	if ( (mouse_pos.x>850) && (mouse_pos.y<256) ) {return;}	// info panel
	
	// delete - sprite, drawing and tile-value			- it would need a further check - if a large object's outer part is clicked, tile value is cleared!!
	if (BuildMode > 0)
	{		
		draw_size = 1;		// can be removed if settings will be available on user interface
		temp_tx = square_x;
		temp_ty = square_y;
		
		// there should be an entity or middle of a larger entity 
		// if no entity, tile value cannot be deleted because it would delete a part of an entity's area, e.g. a tower's blocking area		
		if ( RemoveEntity2( temp_tx , temp_ty ) )  // it modifies draw_size if a large object is selected
			if (draw_size>1) 
			{
				AreaFill(0);
			}
	 		else 
	 		{
	 			TileFill(0);
	 		}
	}	
	
	// replace destination flag
	if (BuildMode==0)
		if ( (square_x!=sx) || (square_y!=sy) )
			if (Tiles[ square_x ][ square_y ]!=1) // tile is passable		// and <> start !!
			{
				// change dest. tile
				fx = square_x;
				fy = square_y;
				// replace 
				var tempa = TileToCoord(fx,GridX);
				var tempb = TileToCoord(fy,GridY);
				me = destobject;
				my.x = tempa;
				my.y = tempb;
				AlignDestFlag();
			}
}

// set proper ActiveTileSquare skin showing actual buildmode
function ChangeMode()
{ 
	if (BuildMode == 0)
	{
		ent_morph(ActiveTileSquare, square2);	// blue - modify path
	}	
	else if (BuildMode == 255)
	{
		ent_morph(ActiveTileSquare, square3);	// red - wait
	}
	else
	{
		ent_morph(ActiveTileSquare, square1);	// green - build
	}	
}

/////////////////////////////////////////
// common after-pathfinding function calls

// reset things after pathfinding
function EndOfPathfinding()
{
	BuildMode = 0; 
	ChangeMode();
	reset(wait_txt,SHOW);
}

// move cost calculation based on actual path and terrain tile values
function Path_Cost_Calculation()
{
	var i,j,k;
	// Tile value conversion to pathfinding move cost values stored by DataArray 
	// it is done in A* but in the same way, but in DirMap it is different, so should be done now
	for(i=0 ; i<GridX ; i++)
		for(j=0 ; j<GridY ; j++)
		{						
			// water - no access
			if (Tiles[i][j] == 1) DataArray[i][j] = 0;   
			
			// grass = normal terrain
			else if (Tiles[i][j] == 0) DataArray[i][j] = GRASS_COST;    
			// stone road
			else if (Tiles[i][j] == 2) DataArray[i][j] = STONE_COST;  
			// dirt road
			else if (Tiles[i][j] == 3) DataArray[i][j] = DIRT_COST;  		
			// woods
			else if (Tiles[i][j] == 4) DataArray[i][j] = WOODS_COST; 
		}  	
	// reset value
	total_move_cost = 0;	
	// add tile values
	for(k=0 ; k<pathlength ; k++)
	{
		i = pathX[k];
		j = pathY[k];
		total_move_cost += DataArray[i][j];		
	}
}

//////////////////////////////////////////
// panel function calls

// control panel 1 functions

// start pathfinding
function call_selected_pathfinder()
{
	while (key_any) {wait (1);}
	
	set(wait_txt,SHOW);
	
	if(BuildMode != 255) 
	{
		if (path_type == 1)
		{
			BuildMode = 255;
			PathFind00();
		}
		else if(path_type == 2) 
		{
			BuildMode = 255;
			PathFind01(1);
		}
		else if(path_type == 3) 
		{
			BuildMode = 255;
			PathFind01(2);
		}
		else if(path_type == 4) 
		{
			BuildMode = 255;
			PathFind01(3);
		}
		else if(path_type == 0) 
		{
			BuildMode = 255;
			PathFind01(0);
		}
		ChangeMode();
	}
}

// select what to display when path is found
function toggle_draw_mode()
{
	drawalltiles = 1 - drawalltiles; // toggle 0-1 -  1 means draw all the little arrows, 0 means only path dots
}

function toggle_depthfirst()
{
	depthfirst = 1 - depthfirst; // toggle 0-1 -  1 means depthfirst, 0 normal A*
}

// select pathfinder
function change_algorythm(button_number)
{
	if (button_number<5) path_type = button_number;
	else path_type = 0;
	
	// reset heuristic values
	if (button_number==1) heur_value=0; 
	else if (button_number==2) heur_value=TEST1_COST;
	else if (button_number==3) heur_value=TEST4_COST;
	else if (button_number==4) heur_value=TEST3_COST;
	else if (button_number==8) heur_value=0;
}

// control panel 2 function - change what to do by mouse
function change_buildmode(button_number)
{	
	if (button_number == 7) 
	{ BuildMode = 0; }					// replace path endpoint models 
	else 
	{ BuildMode = button_number; }	// 1-6 build/delete obstacles
	
	ChangeMode();							// refresh tile pointer square
}

/////////////////////////////////////////////
// create obstacles from a bitmap
function Load_Map()
{
	var i,j;
	var pixel_format;
	var pixel;
	var pixel_alpha;	
	COLOR pixel_color;
	var temp_x, temp_y;
	STRING* temp_str;
	var rnd;
	
	if(BuildMode != 255) 
	{
		ClearMap();
		
		pixel_format  = bmap_lock(obstacle_map,0);				
		for(i=0 ; i<GridX ; i++)
			for(j=0 ; j<GridY ; j++)
			{
				// read pixel
				pixel = pixel_for_bmap( obstacle_map , i , j );	
				// get color and alpha values from pixel
				pixel_to_vec(pixel_color, pixel_alpha, pixel_format, pixel);
				
				// coordinate position
				temp_x = TileToCoord(i,GridX);
				temp_y = TileToCoord(j,GridY);
				// global tile position - used in Align... actions and ...Fill functions
				temp_tx = i;
				temp_ty = j;
				
				// reset buildmode - should be set only if needed in action
				BuildMode = 0;
				// reset draw size to default value
				draw_size = 1;	
				
				// make decision what to build based on pixel color read
				if ( (pixel_color.red == 225) && (pixel_color.green == 225) && (pixel_color.blue == 225) )
				{ 							
					// do nothing - it will be definitely overwritten by a model's settings
				}
				else if ( (pixel_color.red == 0) && (pixel_color.green == 255) && (pixel_color.blue == 0) )
				{ 							
					// set start pos
					sx = i;
					sy = j;
					// player is already created, should be modified only
					me = player;
					my.x = temp_x;
					my.y = temp_y;
					my.pan = 0;
					AlignPlayer();		
					
					// set camera
					CameraObject.x = temp_x - TileSize*2;
					CameraObject.y = temp_y - TileSize*2;
					CamAngle = 225;
				}
				else if ( (pixel_color.red == 255) && (pixel_color.green == 0) && (pixel_color.blue == 0) )
				{ 							
					// set end pos
					fx = i;
					fy = j;
					// flag
					me = destobject;
					my.x = temp_x;
					my.y = temp_y;
					AlignDestFlag();
				}
				else if ( (pixel_color.red == 0) && (pixel_color.green == 0) && (pixel_color.blue == 0) )
				{ 
					// reset to default value
					TileFill(0);
					// build nothing			
				}
				else if ( (pixel_color.red == 255) && (pixel_color.green == 255) && (pixel_color.blue == 255) )
				{ 
					BuildMode = 1; // build water - needed for animation in action called by ent_create
					TileFill(1);								
					// build water tile				
					ent_create( watertile, vector( temp_x , temp_y , 0 ), AlignTile);			
				}
				else if ( (pixel_color.red == 50) && (pixel_color.green == 50) && (pixel_color.blue == 50) )
				{ 
					TileFill(2);				
					// build stoneroad tile				
					ent_create( stoneroadtile, vector( temp_x , temp_y , 0 ), AlignTile);			
				}
				else if ( (pixel_color.red == 100) && (pixel_color.green == 100) && (pixel_color.blue == 100) )
				{ 
					TileFill(3);				
					// build dirtroad tile				
					ent_create( dirtroadtile, vector( temp_x , temp_y , 0 ), AlignTile);			
				}
				else if ( (pixel_color.red == 255) && (pixel_color.green == 255) && (pixel_color.blue == 0) )
				{ 
					TileFill(1);				
					// build wall							
					ent_create( wallmodel, vector( temp_x , temp_y , 0 ), AlignModel);			
				}
				else if ( (pixel_color.red == 0) && (pixel_color.green == 255) && (pixel_color.blue == 255) )
				{ 								
					// build wall		
					draw_size = 3;		// 3x3 area			
					AreaFill(1);
					ent_create( towermodel, vector( temp_x , temp_y , 0 ), AlignModel);							
				}
				else if ( (pixel_color.red == 255) && (pixel_color.green == 0) && (pixel_color.blue == 255) )
				{ 								
					// build tree		
					BuildMode = 6; // needed for creating proper shadows
					draw_size = 5;	// 5x5 area
					AreaFill(4);	// its surroundings have move cost penalty
					TileFill(1);	// the tree itself
					
					random_seed(0); 	// starts with a really random initial value
					rnd = random(2);
					
					if (rnd<1) temp_str = treemodel1;
					else temp_str = treemodel2;	
					ent_create(temp_str, vector( temp_x , temp_y , 0 ), AlignModel);	// the visible model
					
					if (rnd<1) temp_str = treeshadowmodel1;
					else temp_str = treeshadowmodel2;						
					ent_create(temp_str, vector( temp_x , temp_y , 0 ), AlignModel2);	// an invisible model for proper shadow																					
				}
			}
		bmap_unlock(obstacle_map);		// close bmap operations
		BuildMode = 0;  // after bmap loaded reset buildmode
		ChangeMode();
	}
}


function ClearMap()
{
	var i,j;
	var temp_x, temp_y;
					
	if(BuildMode != 255) 
	{
		BuildMode = 255;
		for(i=0 ; i<GridX ; i++)
			for(j=0 ; j<GridY ; j++)
			{
				// coordinate position
				temp_x = TileToCoord(i,GridX);
				temp_y = TileToCoord(j,GridY);
				// global tile position - used in Align... actions and ...Fill functions
				temp_tx = i;
				temp_ty = j;
				// clear existing entities from actual tile anyway
				RemoveEntity(temp_tx,temp_ty);
				RemoveEntity(temp_tx,temp_ty); // because of double model creation of trees
				// clear Tile values
				TileFill(0);			
			}	
		BuildMode = 0;
		ChangeMode();
	}
}
