
// A star pathfinding arrays
// open and closed arrays - can be fused together e.g.: 0-nowhere, 1-openlist, 2-closedlist
BOOL astar_clist[SMALLMAPX][SMALLMAPY];			// var -> char
BOOL astar_olist[SMALLMAPX][SMALLMAPY];			// var -> char
// values needed for distance estimation and tracking
var astar_F[SMALLMAPX][SMALLMAPY];
var astar_G[SMALLMAPX][SMALLMAPY];
var astar_H[SMALLMAPX][SMALLMAPY];
// storing previous path element of each node - it cannot be known during the search where the final path goes
var astar_parent_x[SMALLMAPX][SMALLMAPY];
var astar_parent_y[SMALLMAPX][SMALLMAPY];
///////////////////////////////////////////////////////////////////
// A* pathfinding function declarations - ending with 01
function PathFind01(heuristic);

function Path_Dots01();
function Create_Path_Array01();

function All_Path_Arrows01();
function DrawAllPaths01();
///////////////////////////////////////////////////////////////////

// A* pathfinder main function, including tile value conversion
function PathFind01(heuristic)
{
	// no collision detection needed for path entities
	enable_polycollision = 0; 
	
	// show a panel during calculation
	set( work_in_progress , SHOW );
	//work_in_progress.flags |= SHOW;  
	reset( path_found , SHOW );
	reset( path_error , SHOW );
	wait(5);
	
	time_1 = GetTickCount();
	
	////////// data initialization and path conversion from tile system //////////    
	var i,j, error1, a,b;
	var tempx,tempy, tempa,tempb;
	
	// start and finish tiles are already set, and stored in (sx,sy) and (fx,fy) respectively
	
	// a-star array nulling	
	for(i=0 ; i<GridX ; i++)
		for(j=0 ; j<GridY ; j++)
		{						
			// possible values: 0 or 1
			astar_clist[i][j] = 0;
			astar_olist[i][j] = 0;
			// possible values: >=0
			astar_F[i][j] = 0;
			astar_G[i][j] = 0;
			astar_H[i][j] = 0;
			// possible values: 0..GridX/Y
			astar_parent_x[i][j] = -1;
			astar_parent_y[i][j] = -1;
		}  	
	
	// Tile value conversion to pathfinding move cost values stored by DataArray
	for(i=0 ; i<GridX ; i++)
		for(j=0 ; j<GridY ; j++)
		{						
			// water - no access
			if (Tiles[i][j] == 1) DataArray[i][j] = 0;   
			
			// grass = normal terrain
			else if (Tiles[i][j] == 0) DataArray[i][j] = GRASS_COST;    
			// stone road
			else if (Tiles[i][j] == 2) DataArray[i][j] = STONE_COST;  
			// dirt road
			else if (Tiles[i][j] == 3) DataArray[i][j] = DIRT_COST;  		
			// woods
			else if (Tiles[i][j] == 4) DataArray[i][j] = WOODS_COST; 
		}  	
	
	///////////// start tile settings ////////////////
	
	error1 = 0 ; // no error
	found = 0 ;	// finish not found  
	cycle_time = 0; // init cycle value
	maxpath = MAXPATHLENGTH * 100; // now it is max loop value - to avoid infinite cycle
	
	// put start on open list
	astar_olist[sx][sy] = 1;
	
	// H : give distance between start and finish, 
	// could be a bit fictious but should be proportional with G i.e. path distance walked
	// GRASS_COST is now the value of walk point decrease on grass, as an average value
	// so H and G will based on walk-point penalty values
	
	// H : estimated distance, or "heuristic" function: 
	var test_cost;
	if (heuristic == 1)
	{	
		// MANHATTAN value = ( x-distance + y-distance ) * weight (e.g. an average cost value)
		test_cost = TEST1_COST;
		astar_H[sx][sy] = integer( ( (abs(sx - fx)) + (abs(sy - fy)) ) * test_cost );					
	}
	else if (heuristic == 2)
	{
		// EUCLIDIAN value = sqrt( sqr(x-distance) + sqr(y-distance) ) * weight (e.g. an average cost value)
		test_cost = TEST4_COST;
		astar_H[sx][sy] = integer( sqrt(pow(abs(sx - fx),2) + pow(abs(sy - fy),2)) * test_cost );	
	}
	else if (heuristic == 3)
	{
		// DIAGONAL SHORTCUT value : diagonal distance * 1.4 until fx or fy reached + the remainig orthogonal distance  
		test_cost = TEST3_COST;
		astar_H[sx][sy] = integer( ( (minv( (abs(sx - fx)) , (abs(sy - fy)) ))*1.4 + abs( (abs(sx - fx)) - (abs(sy - fy)) ) ) * test_cost );		
	}
	else astar_H[sx][sy] = 0; // Dijktsra 
	
	// G : path length completed
	astar_G[sx][sy] = 0;
	// F : weighting value - sum of H and G
	astar_F[sx][sy] = astar_H[sx][sy] + astar_G[sx][sy];
	
	cycle_time++;  // just for checking
	
	////////// the main loop ///////////////
	
	while ((!found) && (!error1))
	{
		cycle_time++;
		
		// big value needed for finding the tile having the smallest F value
		a = 10000;
		// tempa and tempb show the tile having the smallest F value
		tempa = -1;		
		tempb = -1;
		// b is >0 if there exists an open list element
		b = 0;
		
		// scan all the tiles
		for(j=0 ; j<GridY ; j++)			
			for(i=0 ; i<GridX ; i++)
			{
				if (astar_olist[i][j]>0) b++;
				
				// looking for open list element having the smallest value
				if ((astar_olist[i][j]>0) && (astar_F[i][j]<=a))
				{
					a = astar_F[i][j]	;
					// tempa-b stores this array position
					tempa = i;
					tempb = j;
				}	
			}
		// now tile [tempa][tempb] holds the best position, and its neighbours will be checked now
		
		// if no open list element , it means error
		if (b == 0) error1++;
		
		// another error situation, if no smallest F value tile determined
		if ((b == 0) && ((tempa<0) || (tempb<0))) error1++;
		
		// the smallest element goes to closed list from open list
		astar_olist[tempa][tempb] = 0;
		astar_clist[tempa][tempb] = 1;
		
		// looking around, i.e. checking the 8 possible neighbours
		// with diagonal travel penalty
		var multiplier1;        // multiply by: 1 if straight, 1.4 if diagonal route

		i = 0;
		while ((i<9) && (!found) && (!error1))
		{
			i++;
			
			// tempx,tempy will hold the array position to be examined
			// S
			if (i == 1)
			{
				tempx = tempa ;
				tempy = tempb -1;
				multiplier1 = 1;
			}
			// E
			else if (i == 2)
			{
				tempx = tempa +1;
				tempy = tempb ;
				multiplier1 = 1;
			}
			// N
			else if (i == 3)
			{
				tempx = tempa ;
				tempy = tempb +1;
				multiplier1 = 1;
			}
			// W
			else if (i == 4)
			{
				tempx = tempa -1;
				tempy = tempb ;
				multiplier1 = 1;
			}			
			// SW
			else if (i == 5)
			{
				tempx = tempa -1;
				tempy = tempb -1;
				multiplier1 = 1.4;
			}
			// SE
			else if (i == 6)
			{
				tempx = tempa +1;
				tempy = tempb -1;
				multiplier1 = 1.4;
			}
			// NE
			else if (i == 7)
			{
				tempx = tempa +1;
				tempy = tempb +1;
				multiplier1 = 1.4;
			}
			// NW
			else if (i == 8)
			{
				tempx = tempa -1;
				tempy = tempb +1;
				multiplier1 = 1.4;
			}			
			// now tempa-b stores the original tile, and tempx-y the neighbour examined actually
				
			
			// needs a safety check of array borders
			if ((tempx >-1) && (tempy>-1) && (tempx<GridX) && (tempy<GridY))
			// neighbour should be accessible, and not on closed list
			if (( DataArray[tempx][tempy] > 0 ) && (!astar_clist[tempx][tempy]))
			{
				if (astar_olist[tempx][tempy])	   // if tile is already on open list
				{
					// check whether now G is smaller than previously i.e. now the path is shorter to here
					// dataarray stores move cost terrain properties
					
					if ( (astar_G[tempa][tempb] + DataArray[tempx][tempy]) < astar_G[tempx][tempy]  )
					{
						// giving new values to F G and "parent" in this tile 
						
						// G = parent value + (current terrain penalty * direction multiplier1) + height penalty
						astar_G[tempx][tempy] = astar_G[tempa][tempb] + (DataArray[tempx][tempy]*multiplier1) + 0 ;  
						// H value must exist because of previous examination is done
						astar_F[tempx][tempy] = astar_G[tempx][tempy] + astar_H[tempx][tempy]; 
						
						astar_parent_x[tempx][tempy] = tempa;
						astar_parent_y[tempx][tempy] = tempb;
					}
				}
				else 		// if tile is not on open list yet
				{
					astar_olist[tempx][tempy] = 1;
					
					astar_G[tempx][tempy] = astar_G[tempa][tempb] + (DataArray[tempx][tempy]*multiplier1) + 0 ;  
					
					if (heuristic == 1)
					{
						// MANHATTAN
						astar_H[tempx][tempy] = integer( ( (abs(tempx - fx)) + (abs(tempy - fy)) ) * test_cost );
					}
					else if (heuristic == 2)
					{
						// EUCLIDIAN
						astar_H[tempx][tempy] = integer( sqrt(pow(abs(tempx - fx),2) + pow(abs(tempy - fy),2)) * test_cost );
					}
					else if (heuristic == 3)
					{
						// DIAGONAL SHORTCUT
						astar_H[tempx][tempy] = integer( ( (minv( (abs(tempx - fx)) , (abs(tempy - fy)) ))*1.4 + abs( (abs(tempx - fx)) - (abs(tempy - fy)) ) ) * test_cost );
					}
					else astar_H[tempx][tempy] = 0; // Dijktsra 
					
					// depth first
					if ( (depthfirst) && (heuristic>0) )
						astar_G[tempx][tempy] = 0;
					
					astar_F[tempx][tempy] = astar_G[tempx][tempy] + astar_H[tempx][tempy] ;
					
					astar_parent_x[tempx][tempy] = tempa;
					astar_parent_y[tempx][tempy] = tempb;
				}				
			}			
		}	// inner while loop
		
		// if finish point is found
		if (astar_olist[fx][fy]) found++;
		
		// to avoid infinite loop, exit while loop
		if ( cycle_time >= maxpath ) error1++; 			
	}	// main while loop
	
	///// error message 
	if ((found<1) || (error1>0))
	{
		set( path_error , SHOW );	
		
		pathlength = 0;
		total_move_cost = 0;
		
		if (drawalltiles) DrawAllPaths01(); // to see where calcucation failed
		
		wait(1);
		wait(-2);
		reset( path_error , SHOW );
		
		EndOfPathfinding(); // allow to do something again		
	}
		
	///// creating path array , showing 2D array positions
	if (found>0)
	{
		// found panel
		set( path_found , SHOW );
		
		// create path array and show path 
		Create_Path_Array01();							
	}   // found>0
	
	// remove panel
	wait(1);
	reset( work_in_progress , SHOW );
	
	// reset collision detection
	enable_polycollision = 1; 
}


//// ------------------------------------

// setting path dot properties
function Path_Dots01()
{
	// horizintal alignment
	vec_set( my.pan , vector(270,90,0) );       	
	// scale to actual tilesize
	my.scale_x = TileSize/GRIDSIZE;
	my.scale_y = TileSize/GRIDSIZE;
	
	my.z = 0 + PathLevel;	
	// set visibility things
	set( me , PASSABLE | ZNEAR );
	my.ambient = 100;  
	my.emask &= ~DYNAMIC;
}


// could be included into main pathfinding function 
function Create_Path_Array01()
{
	var i,j, k, tempx,tempy, tempa,tempb, counter,counterm;
	
	//// clearing existing path data ////
	for(i=0 ; i<MAXPATHLENGTH ; i++)
	{
		// pathX and Y store x and y array-coordinates within the range 0..999, so 1000 is the value of no path elements
		pathX[i] = 1000;
		pathY[i] = 1000;
		// pathDir can show in 8 directions or nowhere=0 . it is needed for representing the path by small arrows
		pathDir[i] = 0;
	}
	
	// temporarily the first element of path-array is the finish node
	counter = 0;			
	pathX[counter] = fx;
	pathY[counter] = fy; 
	maxpath = MAXPATHLENGTH;     // now it shows the global maximum value given 
	
	///// looking for start node backward from finish /////
	while ( ((pathX[counter] != sx) || (pathY[counter] != sy)) && (counter<(maxpath-1)) )
	//for(i=0 ; (i<(maxpath-2)) ; i++ )
	{
		// counter now points at the current element
		counter += 1;
		counterm = counter - 1;
		
		// pointing at the previous elment's position
		tempx = pathX[counterm] ;
		tempy = pathY[counterm] ;
		
		if ((astar_parent_x[ tempx ][ tempy ] < 1000) && (astar_parent_y[ tempx ][ tempy ] < 1000))   //not needed
		{
			// the new element is the parent of the previous element
			pathX[counter] = astar_parent_x[ tempx ][ tempy ] ;
			pathY[counter] = astar_parent_y[ tempx ][ tempy ] ;
		}
	} // while
	
	// error - too long path
	if (counter > (maxpath - 2))
		set( path_error , SHOW );
	
	// now it shows the real path length
	maxpath = counter +1;	
	
	////// finalizing the path //////
	tempx = 0;
	tempy = 0;
	var maxpathlength_minus1_minusi;
	for(i=0 ; i<maxpath ; i++)               
	{
		maxpathlength_minus1_minusi = maxpath -1 -i ;
		// last element to temp
		tempx = pathX[ maxpathlength_minus1_minusi ] ;
		tempy = pathY[ maxpathlength_minus1_minusi ] ;
		// first element to last
		pathX[ maxpathlength_minus1_minusi ] = pathX[ i ];
		pathY[ maxpathlength_minus1_minusi ] = pathY[ i ];
		// temp element to first
		pathX[ i ] = tempx;
		pathY[ i ] = tempy;
	}
	// pathX and Y are ready to use
	
	// get time length of path calculation, and without path display
	time_2 = GetTickCount();
	time_elapsed = (time_2 - time_1)/1000;
	str_for_num(timer_value, time_elapsed);

	///// representing the path //////
	
	// grid origo offset, if vertex 0,0,0 is the middle of the grid
	tempx = (GridX / 2) * TileSize - (TileSize / 2);  // if 40 then 1248
	tempy = (GridY / 2) * TileSize - (TileSize / 2);  // if 20 then 672
	
	k = 0;
	for( i=0 ; i<maxpath ; i++ )
	{
		if ( (pathX[i] < 1000) && (pathY[i] < 1000) )
		{   		 
			// reverse calculation from node point to coordinate values
			tempa = ( pathX[i] * TileSize ) - tempx ;
			tempb = ( pathY[i] * TileSize ) - tempy ; 
			
			PathArray[k] = ent_create( path_dot , vector(tempa,tempb,0) , Path_Dots01 );
			
			k++;
		}
	}	
	
	// display path length
	pathlength = maxpath;
	
	// representing all search values, just for fun
	if (drawalltiles) DrawAllPaths01();
		
	// removing the little path dots
	wait(1);
	wait(-10);
	reset( path_found , SHOW );
	
	for(i=0 ; i<k ; i++)
		ptr_remove( PathArray[i] );
	wait(1);
	
	// get path cost
	Path_Cost_Calculation();
	EndOfPathfinding(); // allow to do something again	
}


////-----------------------------------------

// setting direction signs' position
function All_Path_Arrows01()
{
	// horizintal alignment, shold be panned by 270 and tilt by 90
	vec_set( my.pan , vector(270,90,0) );      
	// scale to actual tilesize
	my.scale_x = TileSize/GRIDSIZE;
	my.scale_y = TileSize/GRIDSIZE;
	
	my.z = 0 + PathLevel + RoadLevel;	

	set( me , PASSABLE );		
	my.ambient = 100;  	
	my.emask &= ~DYNAMIC;
}


// drawing directions stored in DataArray showing all possible path-directions from start point
function DrawAllPaths01()
{
	var i,j, k;
	var tempx, tempy, tempa, tempb;

	// grid origo offset, if vertex 0,0,0 is the middle of the grid
	tempx = (GridX / 2) * TileSize - (TileSize / 2);  // if 40 then 1248
	tempy = (GridY / 2) * TileSize - (TileSize / 2);  // if 20 then 672

	k = 0; // entity array counter
	for(j=0 ; j<GridY; j++)
	for(i=0 ; i<GridX; i++)
	{
		// x;y coordinates
		tempa = ( i * TileSize ) - tempx ;
		tempb = ( j * TileSize ) - tempy ;
		
		// if accessible
		if ( DataArray[i][j] > 0 )
		{
			// if not start - only start has no parent so finish can be drawn with direction arrow too
			if (( i!= sx ) || ( j!= sy )) 
			{
				// no value calculated  =-1
				if ((astar_parent_x[i][j] < 0) && (astar_parent_y[i][j] < 0))
					{ ArrowArray[k] = ent_create( "0.tga" , vector(tempa,tempb,0) , All_Path_Arrows01 ); }	
				else
				{
					// direction values
					if ((astar_parent_x[i][j] == (i)) && (astar_parent_y[i][j] == (j+1)))
						{ ArrowArray[k] = ent_create( "1.tga" , vector(tempa,tempb,0) , All_Path_Arrows01 ); }
					else if ((astar_parent_x[i][j] == (i-1)) && (astar_parent_y[i][j] == (j)))
						{ ArrowArray[k] = ent_create( "2.tga" , vector(tempa,tempb,0) , All_Path_Arrows01 ); }
					else if ((astar_parent_x[i][j] == (i)) && (astar_parent_y[i][j] == (j-1)))
						{ ArrowArray[k] = ent_create( "3.tga" , vector(tempa,tempb,0) , All_Path_Arrows01 ); }
					else if ((astar_parent_x[i][j] == (i+1)) && (astar_parent_y[i][j] == (j)))
						{ ArrowArray[k] = ent_create( "4.tga" , vector(tempa,tempb,0) , All_Path_Arrows01 ); }
					
					else if ((astar_parent_x[i][j] == (i+1)) && (astar_parent_y[i][j] == (j+1)))
						{ ArrowArray[k] = ent_create( "5.tga" , vector(tempa,tempb,0) , All_Path_Arrows01 ); }
					else if ((astar_parent_x[i][j] == (i-1)) && (astar_parent_y[i][j] == (j+1)))
						{ ArrowArray[k] = ent_create( "6.tga" , vector(tempa,tempb,0) , All_Path_Arrows01 ); }
					else if ((astar_parent_x[i][j] == (i-1)) && (astar_parent_y[i][j] == (j-1)))
						{ ArrowArray[k] = ent_create( "7.tga" , vector(tempa,tempb,0) , All_Path_Arrows01 ); }
					else if ((astar_parent_x[i][j] == (i+1)) && (astar_parent_y[i][j] == (j-1)))
						{ ArrowArray[k] = ent_create( "8.tga" , vector(tempa,tempb,0) , All_Path_Arrows01 ); }
				} // else
			}										
		} // accessible		 
		// if no access
		else
//		if ( DataArray[i][j] == 0 )
		{ 
			ArrowArray[k] = ent_create( "11.tga" , vector(tempa,tempb,10) , All_Path_Arrows01 ); 
		}
			
		k++;
	}		
	
	// start or destination
	ArrowArray[k] = ent_create( "9.tga" , vector(sx,sy,0) , All_Path_Arrows01 ); 
	k++;
	ArrowArray[k] = ent_create( "10.tga" , vector(fx,fy,0) , All_Path_Arrows01 ); 	
	k++;
	
	// removing the little arrows showing all possible directions
	wait(1);
	wait(-5);
	for(i=0 ; i<k ; i++)
		ptr_remove( ArrowArray[i] );
}


