///////////////////////////////
#include <acknex.h>
#include <default.c>
#include <d3d9.h>		// for antialiasing
//#include <level.c>		// splash screen and fast terrain height calculator helper functions - not used
#include <mtlFX.c>		// for terraintex.fx multiple detail skinning effect (not shader, works in free edition too)
#include <windows.h>		// to access GetTickCount for timer
///////////////////////////////
// standar map size definitions:
#define SMALLMAPX 128				// should be even number (in case of odd number path array is represented wrongly)
#define SMALLMAPY 128				// should be even number (in case of odd number path array is represented wrongly)
#define SMALLMAPSIZE 16384			// 2D array element number SMALLMAPX * SMALLMAPY
//#define SMALLMAPSIZE3 49152		
#define MAXPATHLENGTH 1000			// should be multiple of map array size, but heavily depends on map complexity
#define GRIDSIZE 64.0				// basic grid value, used for tile creation, and needed for scaling down tile sized graphic entities e.g. if 32 is used as tile size
////////////////////////////////
// skill naming 
#define ANIMATION skill21			// anim frame
#define XTILE_NUMBER skill22		// 0..127 x pos
#define YTILE_NUMBER skill23		// 0..127 y pos
#define ZTILE_NUMBER skill24		// 0..2   z pos
#define AREA_SIZE  skill25			// area occupied by entity in tiles: 1x1 tile = 1, 2x2 tiles = 2 ...
#define ENTITY_TYPE  skill26		// differentiate between obstacle sprites/models, and other system entities. under 100 it is an deletable obstacle
////////////////////////////////////
// walk point penalties on different terrains:
// 0..24 flat land
#define WATER_COST 5			// TILE VALUE : 1 - no access, used only for ships, deeper than 1 tile depth
#define STONE_COST 4			// TILE VALUE : 2
#define DIRT_COST 6			// TILE VALUE : 3
#define GRASS_COST 10		// TILE VALUE : 0 - average value, it has significant influence on A* remaining distance estimation (= heuristic function)
#define WOODS_COST 16		// TILE VALUE : 4
#define SNOW_COST 18			// TILE VALUE : 5 - not used
#define SAND_COST 20			// TILE VALUE : 6 - not used
#define SWAMP_COST 30		// TILE VALUE : 7 - water with less than 1 tile depth
// 25..49 various building models are there 									
#define ONWALL_COST 5		// TILE VALUE : MODELS+0 wall (z=3), should be 4 later, but set by a defined constant
									// TILE VALUE : MODELS+1 inside tower (z=3)
#define ONRUINS_COST 25		// TILE VALUE : MODELS+2 - not used
#define ONBRIDGE_COST 10	// TILE VALUE : MODELS+3 - not used
#define STAIRS_COST 17		// TILE VALUE : MODELS+20 - not used
#define LADDER_COST 24		// TILE VALUE : MODELS+24 ladder (z=1.5)
//////////////// 
// map modifiers : 0.00 format is needed when they are in calculations where digits should be kept
#define MODELS 25								// buildings are marked over this tile value  
#define HEIGHT_OFFSET 50					// base value plus this number means rougher terrain, plus 2 times this value means much rougher
#define MAX_HEIGHT_DIFFERENCE 3.00		// max z difference (given in tilesize) between 2 adjacent tiles that is still walkable by a unit
#define VERTICAL_LEVELS 3					// 0-1-2 levels exist: gound floor, 1st floor, and 2nd floor
// height difference ADDITIONAL costs - should be added to flat land cost - 25 should be enough
#define HILL_COST 10			// TILE VALUE : base +1*HEIGHT_OFFSET 			// or could be used only at tile to move cost transformation
#define MOUNT_COST 20		// TILE VALUE : base +2*HEIGHT_OFFSET - not used, HILL_COST is multiplied
#define ABYSS_COST 0			// TILE VALUE : 1 - no access
/////////////////////////////////////
// heuristic functions' estimated cost values - based on experience so play with them bravely! - good result heavily depends on actual terrain
#define TEST1_COST 6.75		// MANHATTAN - should be a bit lower than average value to give a good path
#define TEST2_COST 10.0		// average value or higher can cause over-estimation - bad results, shows simply the shortest path without taking into account move cost penalties
#define TEST3_COST 9.0		// DIAGONAL SHORTCUT - should be close to average cost value
#define TEST4_COST 9.5		// EUCLIDIAN - should be a nearly average cost value
//////////////////////////////

#include "maphandling_3D.c"
#include "pathdirmap.c"
#include "pathastar.c"
#include "pathastar_turn.c"
#include "objecthandling.c"
#include "panelhandling.c"

///////////////////////////////

// terrain skin material settings
MATERIAL* terrain_mat =
{
       effect = "terraintex.fx"; // no shader so it works with free edition too
}
action terrain_action()
{
   my.ENTITY_TYPE = 1000;    // to avoid deletion
//   my.scale_z = 5;
	my.material = terrain_mat;
}
action water_action()
{
//	my.scale_x = 8; 
//	my.scale_y = 8;
	my.ENTITY_TYPE = 1000;    // to avoid deletion
	set(my,PASSABLE | TRANSLUCENT );	
	my.alpha=75;
	// basic water animation
	var i = 0;
	while(1)
	{	
		// waves by texture shift
		my.v -= 0.05*cosv(i)*time_step;
		my.u -= 0.05*time_step;						
		// waves by z move
		my.z = i/4;  
		i += 1;
		i %= 360;			
		wait(1);
	}
}
///////////////////////     
// to remap f1-f12 
function key_do_nothing() 
{
	wait(1);
}

function main()
{
	// avoid 3dgs calls
	on_f1  = key_do_nothing;
	on_f2  = key_do_nothing;
	on_f3  = key_do_nothing;
	on_f4  = key_do_nothing;
	on_f5  = key_do_nothing;
	on_f6  = key_do_nothing;
	on_f7  = key_do_nothing;
	on_f8  = key_do_nothing;
	on_f9 = key_do_nothing;
	on_f10 = key_do_nothing;
	on_f12 = key_do_nothing;
	on_0 = key_do_nothing;
	
	fps_max = 50; // limit the frame rate
	video_mode = 8; // run in 1024x768 pixels
	video_depth = 32; // start the engine in 32 bit mode
	video_screen = 1; // start in full screen mode
	
	d3d_antialias = 1;
	preload_mode = 3+4; 
	
	// setting sky color
	sky_color.red = 25;
	sky_color.green = 125;
	sky_color.blue = 175; 
   
	// level load
	max_entities = SMALLMAPX * SMALLMAPY * 5 + MAXPATHLENGTH * 10 + 500;    // reserve enough memory for entities representing the path arrays
//	terrain_chunk=0;		// needed only if more sewed terrains are used 
	level_load(NULL);
	
	// setting sun
	vec_set(sun_color,vector(150,175,200));  
	sun_light = 60;    
	sun_angle.pan = 315; // from South-East
   sun_angle.tilt = 60; 
   
   // setting dynamic shadow type
	mat_shadow.alpha = 35;
	shadow_stencil = 1;
	
	wait(1);
	
	detail_size = -12;	
	terrain_entity = ent_create( "3D.hmp" , vector(0,0,0), terrain_action);	
	detail_size = -6;
	ent_create( "water2.hmp" , nullvector, water_action);					
	wait(1);	
	
	// call level settings initialization and handling functions
	Interface_Init();
	
while(1)
        {       		        		
        		wait(1);
        }
	
}