// tile objects 
var water_anim = 1; 										// global variable to synchronize water animation sprites to keep tileable placement
STRING* watertile = "water3+7.tga";					// water sprite - with 7 frame
STRING* dirtroadtile = "earthtile.tga";			// dirt road sprite
STRING* stoneroadtile = "stoneroad.tga";			// stone road sprite
//STRING* woodstile = "grass.tga";					// grass for woods sprite
STRING* wallmodel = "wall01.mdl";					// 1 tile obstacle - because of shadow, mdl is needed - wmb has no shadow
STRING* towermodel = "closedtower04.mdl";			// 3x3 tile sized obstacle
STRING* bastionmodel = "tower04.mdl";				// 3x3 tile sized obstacle
STRING* treemodel1 = "tree003dds.mdl";				// 1 tile obstacle
STRING* treeshadowmodel1 = "tree003tga_b.mdl";	// 1 tile obstacle used only for casting tree shadow
STRING* treemodel2 = "tree002dds.mdl";				// 1 tile obstacle
STRING* treeshadowmodel2 = "tree002tga_b.mdl";	// 1 tile obstacle used only for casting tree shadow
STRING* laddermodel1 = "ladder01.mdl";				// 1 tile obstacle, move between Z levels

//////////////////////////////////////////
// object removal 
function RemoveEntity(tempx,tempy);
function RemoveEntity2(tempx,tempy);

// tile value handling
//function TileFill(num,z);
//function AreaFill(num,z);
function TileFill(num1,num2,num3,z1,z2);
function AreaFill(num1,num2,num3,z1,z2);

// object sprite and model settings
action AlignTile();
action AlignModel();
action AlignModel2();

// object placing/removing
function PlaceObject();
function PlaceObject2();

function RotateEntity(tempx,tempy);

/////////////////////////////////////////
// object removal

// remove entity from the tile given
function RemoveEntity(tempx,tempy)
{
	you = ent_next(NULL); // get first entity
 	while (you) // repeat until there are no more entities
	{ 
		// check position stored in entity skills
		if ( (you.XTILE_NUMBER==tempx) && (you.YTILE_NUMBER==tempy) )
			if (you.ENTITY_TYPE < 100)
			{
				ent_remove(you);  			
				break;			// end of loop
			}				
 		you = ent_next(you); // get next entity
	}	
}

// remove entity from the tile given with feedback about its type, and detect its draw size enabling to clear all the tile values in the surroundings
function RemoveEntity2(tempx,tempy)
{
	var temp=0;
	
	you = ent_next(NULL); // retrieve first entity
 	while (you) // repeat until there are no more entities
	{ 
		if ( (you.XTILE_NUMBER==tempx) && (you.YTILE_NUMBER==tempy) )
			if (you.ENTITY_TYPE < 100)
			{
				draw_size = you.AREA_SIZE;  // used for removing a large-area-entity occupying more tile , those should be cleared all, after this function terminates
				temp = you.ENTITY_TYPE;
				ent_remove(you);  				
				return(temp);	// exit and feedback: object deleted		
			}				
 		you = ent_next(you); // get next entity
	}
	return(0);	// feedback: nothing was deleted
}

/////////////////////////////////////////////////
// writing new values into Tiles array

// filling the actual tile
function TileFill(num1,num2,num3,z1,z2)
{                                     		
	Tiles[ temp_tx ][ temp_ty ][ 0 ] = num1;	// set ground/model type
	Tiles[ temp_tx ][ temp_ty ][ 1 ] = num2;	// set ground/model type
	Tiles[ temp_tx ][ temp_ty ][ 2 ] = num3;	// set ground/model type
	
	TilesZ[ temp_tx ][ temp_ty ][ 0 ] = z1;	// set model height
	TilesZ[ temp_tx ][ temp_ty ][ 1 ] = z2;	// set model height
	
	SetHeightOffset(temp_tx , temp_ty);	// reset slope and depth properties and writes into ground level tile value: Tiles[x][y][0]
}
 
 // filling more than 1 tiles, more exactly: draw_size x draw_size tiles
function AreaFill(num1,num2,num3,z1,z2)
{                                     
	var i, j;
	var tempa, tempb;           

	tempa = temp_tx - integer(draw_size/2) ;
	tempb = temp_ty - integer(draw_size/2) ;
		
	for (i=0 ; i< (draw_size) ; i++)	
		for (j=0 ; j< (draw_size) ; j++)
			{
				// preventing from trying overwriting tiles out of grid boundaries
				if ( (tempa+i>=0)  && (tempa+i<GridX) && (tempb+j>=0) && (tempb+j<GridY) )
				{	
					Tiles[ tempa+i ][ tempb+j ][ 0 ] = num1;	// set ground/model type
					Tiles[ tempa+i ][ tempb+j ][ 1 ] = num2;	// set ground/model type
					Tiles[ tempa+i ][ tempb+j ][ 2 ] = num3;	// set ground/model type
					
					TilesZ[ tempa+i ][ tempb+j ][ 0 ] = z1;	// set model height
					TilesZ[ tempa+i ][ tempb+j ][ 1 ] = z2;	// set model height
					
					SetHeightOffset(tempa+i , tempb+j);	// reset slope and depth properties and writes into Tiles
					
					// if there is any other entity in the building area - remove it, except the placement tile
//					if ( !((tempa+i == temp_tx) && (tempb+j == temp_ty)) )	// exclusion not needed - now this function is called always before entity creation
						
					RemoveEntity(tempa+i,tempb+j);
				}
			}
}

////////////////////////////////////////
// obstacle creation functions

// setting a sprite
action AlignTile()
{	
	my.XTILE_NUMBER = temp_tx;
	my.YTILE_NUMBER = temp_ty;
	my.ZTILE_NUMBER = temp_tz;
	my.AREA_SIZE = 1;
	my.ENTITY_TYPE = 1;
	
	// horizintal alignment
	vec_set( my.pan , vector(270,90,0) );       
	// a bit over surface but under the mouse following square sprite
	my.z += RoadLevel;
	// scale to actual tilesize
	my.scale_x = 2 * TileSize/GRIDSIZE ;
	my.scale_y = 2 * TileSize/GRIDSIZE ;
	// the same snap function  
	SnapIt();
	// check z
	my.z = TilesMax[temp_tx][temp_ty] + RoadLevel;
	
	my.ambient = 100;
	set( me , PASSABLE | SHADOW);		// to avoid collision
	
	// water animation - all is synchronized by water_anim global variable to be tileable making rivers
	if (BuildMode == 1)
	{
		// animating water flow
		while(1)
		{
	      my.frame = water_anim;  
			wait(1);
		}
	}
}

// making the touched model transparent - easier obstacle management
function MouseEvent()
{
	switch (event_type)
  	{
  		case EVENT_TOUCH:
  			set(my, TRANSLUCENT );  //   
			my.alpha = 33;
			return;
		case EVENT_RELEASE:
			reset(my, TRANSLUCENT );  //   
			my.alpha = 100;
			return;
	}	
}

// setting a model
action AlignModel()
{	
	my.XTILE_NUMBER = temp_tx;
	my.YTILE_NUMBER = temp_ty;
	my.ZTILE_NUMBER = temp_tz;
	my.AREA_SIZE = draw_size;	
	my.ENTITY_TYPE = 2;
	
	// scale to actual tilesize
	my.scale_x = TileSize/GRIDSIZE;
	my.scale_y = TileSize/GRIDSIZE;
	my.scale_z = TileSize/GRIDSIZE;
	
	c_setminmax(me);
	VECTOR vFeet;
	vec_for_min(vFeet,me); // vFeet.z = distance from player origin to lowest vertex
	my.z -= vFeet.z +5;	
	SnapIt();
	// check z
	my.z = TilesMin[temp_tx][temp_ty] - vFeet.z;
	
	set( me , POLYGON);	
	
	// tree secial settings
	if (BuildMode != 6) set( me , SHADOW );
	if (BuildMode == 6) set( me , FLAG2 );		// to exclude vegetation from some c_tracing that determines both terrain and building top surface
	
	// ladder special settings
	if ((BuildMode >= 96) && (BuildMode <= 99))
	{
		my.pan += 90*(BuildMode-96);		
		my.ENTITY_TYPE = 3+(BuildMode-96);
	}
	// tower special settings
	if ((BuildMode >= 92) && (BuildMode <= 95))
	{
		my.pan += 90*(BuildMode-92);		
		my.ENTITY_TYPE = 7+(BuildMode-96);
	}
	
	my.ambient = 100;
	my.emask |= ENABLE_TOUCH | ENABLE_RELEASE;
	my.event = MouseEvent;
}

// setting a shadow maker model of another model having transparent skin thus casting no shadow
action AlignModel2()
{	
	my.XTILE_NUMBER = temp_tx;
	my.YTILE_NUMBER = temp_ty;
	my.ZTILE_NUMBER = temp_tz;
	my.AREA_SIZE = draw_size;	
	my.ENTITY_TYPE = 3;
	
	// scale to actual tilesize
	my.scale_x = TileSize/GRIDSIZE;
	my.scale_y = TileSize/GRIDSIZE;
	my.scale_z = TileSize/GRIDSIZE;
	
	VECTOR vFeet;
	vec_for_min(vFeet,me); // vFeet.z = distance from player origin to lowest vertex
	my.z -= vFeet.z +5;	
	SnapIt();
	// check z
	my.z = TilesMin[temp_tx][temp_ty] - vFeet.z;
	
	my.ambient = 100;
	set( me , SHADOW | INVISIBLE | FLAG2 );		// used for casting a shadow of a tree model having transparent skin, and also to avoid some c_tracings
	
	my.emask |= ENABLE_TOUCH | ENABLE_RELEASE;
	my.event = MouseEvent;
}

///////////////////////////////////////////////////
// user mouse operations

// left click operations
function PlaceObject()
{	
	STRING* temp_str;
	var rnd;
	
	// if  mouse is over a panel then exit this function
	if ( (mouse_pos.x<256) && (mouse_pos.y>512) ) {return;}	// control panel 2
	if ( (mouse_pos.x>768) && (mouse_pos.y>512) ) {return;}	// control panel 1
	if ( (mouse_pos.x>850) && (mouse_pos.y<256) ) {return;}	// info panel
	
	// replace player model
	if (BuildMode == 0)
		if ( (square_x!=fx) || (square_y!=fy) || (square_z!=fz) )
			if (Tiles[ square_x ][ square_y ][square_z]!=1) 		// tile is passable		// and <> start !!
			{
				// change start tile
				sx = square_x;
				sy = square_y;
				sz = square_z;
				// replace 
				var tempa = TileToCoord(sx,GridX);
				var tempb = TileToCoord(sy,GridY);
				me = player;
				my.x = tempa;
				my.y = tempb;
				AlignPlayer();					
			}
	
	// build
	if ((BuildMode > 0))					// && (BuildMode < 255)
	{
		if ( Tiles[ square_x ][ square_y ]%HEIGHT_OFFSET==0 )		// if tile is empty
		{
			// global temporary variables used in "Align-Tile/Model/Model2" actions and "Tile/Area-Fill" functions
			temp_tx = square_x;
			temp_ty = square_y;
			temp_tz = square_z;	// it has no real meaning - no building onto building placement
			
			Mouse3D(); // detect again mouse position on map
			if (HIT_TARGET) // do drawing only if mouse points to terrain - else target has a value of 0,0 which would result in drawing at Node (0,0)
			{					
				// sprite creation - any free tile							
				// water - no access - sprite creation
				if (BuildMode == 1) 
				{
					if ( !((temp_tx==sx) && (temp_ty==sy)) && !((temp_tx==fx)  && (temp_ty==fy)) )
					{
						TileFill(1,1,1,0,0);					
						ent_create(watertile, target, AlignTile);
					}
				}				
				// road - sprite creation
				else if (BuildMode == 2) 
				{
					TileFill(2,1,1,0,0);					
					ent_create(stoneroadtile, target, AlignTile);
				}					
				else if (BuildMode == 3) 
				{
					TileFill(3,1,1,0,0);
					ent_create(dirtroadtile, target, AlignTile);					
				}	
				
				// model creation - free tile if neither start nor end of path 
				if ( !((temp_tx==sx) && (temp_ty==sy)) && !((temp_tx==fx)  && (temp_ty==fy)) )		
				{				
					// 1x1 wall
					if (BuildMode == 4) 
					{
						draw_size = 1;		// its draw area size in tiles
						TileFill(1,MODELS+0,1,3,0);
						ent_create(wallmodel, target, AlignModel);					
					}				
					// 3x3 tower - only inside 1st floor is walkable
					else if (BuildMode == 5) 
					{
						draw_size = 3;		// its draw area size in tiles
						AreaFill(1,MODELS+1,1,3,0);
						ent_create(towermodel, target, AlignModel);		
						// only middle and doors are passable
						Tiles[temp_tx-1][temp_ty+1][1]=1;
						Tiles[temp_tx+1][temp_ty+1][1]=1;
						Tiles[temp_tx-1][temp_ty-1][1]=1;
						Tiles[temp_tx+1][temp_ty-1][1]=1;
						Tiles[temp_tx][temp_ty-1][1]=1;
						Tiles[temp_tx+1][temp_ty][1]=1;
					}				
					// 5x5 tree
					else if (BuildMode == 6) 
					{
						draw_size = 5;		// its draw area size in tiles
						AreaFill(4,1,1,0,0);		// around the tree thearea is woods
						TileFill(1,1,1,0,0);		// the tree itselfis an obstacle	
						
						random_seed(0); 	// starts with a really random initial value
						rnd = random(2);
						
						if (rnd<1) temp_str = treemodel1;
						else temp_str = treemodel2;	
						ent_create(temp_str, target, AlignModel);	// the visible model
						
						if (rnd<1) temp_str = treeshadowmodel1;
						else temp_str = treeshadowmodel2;						
						ent_create(temp_str, target, AlignModel2);	// an invisible model for proper shadow					
					}
					
					// ...
					
					// 3x3 tower with ladder -------------->>> it should be 2 models : 1) outer with FLAG2, 2) 1st and 2nd floor and ladder - it can be c-traced only to determine proper level easily
					else if ((BuildMode >= 92) && (BuildMode <= 95))
					{
						draw_size = 3;		// its draw area size in tiles
						AreaFill(1,MODELS+1,MODELS+1,3,3.5);
						ent_create(bastionmodel, target, AlignModel);		
						
						// tile value modifications:
						
						// 1st floor: corners and middle impassable (middle ladder corresponds to 2nd floor)
						Tiles[temp_tx-1][temp_ty+1][1]=1;
						Tiles[temp_tx+1][temp_ty+1][1]=1;
						Tiles[temp_tx-1][temp_ty-1][1]=1;
						Tiles[temp_tx+1][temp_ty-1][1]=1;
						Tiles[temp_tx][temp_ty][1]=1;
						
						// 2nd floor ladder height settings
						TilesZ[temp_tx][temp_ty][1]-=1.75;
						
						// doors and ladder accessibility depend on tower orientation
						if (BuildMode==92) 
						{
							// 1st floor settings
							Tiles[temp_tx][temp_ty-1][1]=1;
							Tiles[temp_tx+1][temp_ty][1]=1;
							Tiles[temp_tx-1][temp_ty][1]+=0.2;							
							// 2nd floor settings
							Tiles[temp_tx][temp_ty][2]+=0.45;
						}
						else if (BuildMode==93) 
						{
							// 1st floor settings
							Tiles[temp_tx][temp_ty+1][1]=1;
							Tiles[temp_tx+1][temp_ty][1]=1;
							Tiles[temp_tx][temp_ty-1][1]+=0.3;							
							// 2nd floor settings
							Tiles[temp_tx][temp_ty][2]+=0.15;
						}
						else if (BuildMode==94) 
						{
							// 1st floor settings
							Tiles[temp_tx][temp_ty+1][1]=1;
							Tiles[temp_tx-1][temp_ty][1]=1;
							Tiles[temp_tx+1][temp_ty][1]+=0.4;							
							// 2nd floor settings
							Tiles[temp_tx][temp_ty][2]+=0.25;
						}
						else if (BuildMode==95) 
						{
							// 1st floor settings
							Tiles[temp_tx][temp_ty-1][1]=1;
							Tiles[temp_tx-1][temp_ty][1]=1;
							Tiles[temp_tx][temp_ty+1][1]+=0.1;							
							// 2nd floor settings
							Tiles[temp_tx][temp_ty][2]+=0.35;
						}	
					}	
					// ladder - limited placement: entrance should be on accessible ground
					if ((BuildMode >= 96) && (BuildMode <= 99))
					if ( 
					( (BuildMode==96) && 	(Tiles[temp_tx][temp_ty-1][0]%HEIGHT_OFFSET<MODELS) && (Tiles[temp_tx][temp_ty-1][0]%HEIGHT_OFFSET!=1) )
					|| ( (BuildMode==97) && (Tiles[temp_tx+1][temp_ty][0]%HEIGHT_OFFSET<MODELS) && (Tiles[temp_tx+1][temp_ty][0]%HEIGHT_OFFSET!=1) )
					|| ( (BuildMode==98) && (Tiles[temp_tx][temp_ty+1][0]%HEIGHT_OFFSET<MODELS) && (Tiles[temp_tx][temp_ty+1][0]%HEIGHT_OFFSET!=1) )
					|| ( (BuildMode==99) && (Tiles[temp_tx-1][temp_ty][0]%HEIGHT_OFFSET<MODELS) && (Tiles[temp_tx-1][temp_ty][0]%HEIGHT_OFFSET!=1) )
					)
					{
						draw_size = 1;		// its draw area size in tiles
						TileFill(1,MODELS+24,1,1.5,0);
						
						if (BuildMode==96) 
						{							
							Tiles[temp_tx][temp_ty-1][0]+=0.3;
							
							// 1st floor settings
							Tiles[temp_tx][temp_ty][1]+=0.15;
						}
						else if (BuildMode==97) 
						{
							Tiles[temp_tx+1][temp_ty][0]+=0.4;
							
							// 1st floor settings
							Tiles[temp_tx][temp_ty][1]+=0.25;
						}
						else if (BuildMode==98) 
						{
							Tiles[temp_tx][temp_ty+1][0]+=0.1;
							
							// 1st floor settings
							Tiles[temp_tx][temp_ty][1]+=0.35;
						}
						else if (BuildMode==99) 
						{
							Tiles[temp_tx-1][temp_ty][0]+=0.2;
							
							// 1st floor settings
							Tiles[temp_tx][temp_ty][1]+=0.45;
						}
						
						ent_create(laddermodel1, target, AlignModel);					
					}	
				}
			}
		}		     
	}    

}

// right click operations
function PlaceObject2()
{
	var temp = 0;
	
	// if  mouse is over a panel then exit this function
	if ( (mouse_pos.x<256) && (mouse_pos.y>512) ) {return;}	// control panel 2
	if ( (mouse_pos.x>768) && (mouse_pos.y>512) ) {return;}	// control panel 1
	if ( (mouse_pos.x>850) && (mouse_pos.y<256) ) {return;}	// info panel
	
	// delete - sprite, drawing and tile-value			- it would need a further check - if a large object's outer part is clicked, tile value is cleared!!
	if (BuildMode > 0)
	{		
		draw_size = 1;		// can be removed if settings will be available on user interface
		temp_tx = square_x;
		temp_ty = square_y;
		temp_tz = square_z;
		
		// there should be an entity or middle of a larger entity 
		// if no entity, tile value cannot be deleted because it would delete a part of an entity's area, e.g. a tower's blocking area	
		temp = RemoveEntity2( temp_tx , temp_ty );	
		if ( temp )  // it modifies draw_size if a large object is selected
			if (draw_size>1) 
			{
				AreaFill(0,1,1,0,0);
			}
	 		else 
	 		{
	 			TileFill(0,1,1,0,0);
	 			
	 			// ground floor ladder entrance deletion
 				if (temp==3) 		{Tiles[temp_tx][temp_ty-1][0] = integer(Tiles[temp_tx][temp_ty-1][0]); }
	 			else if (temp==4) {Tiles[temp_tx+1][temp_ty][0] = integer(Tiles[temp_tx+1][temp_ty][0]); }
	 			else if (temp==5) {Tiles[temp_tx][temp_ty+1][0] = integer(Tiles[temp_tx][temp_ty+1][0]); }
	 			else if (temp==6) {Tiles[temp_tx-1][temp_ty][0] = integer(Tiles[temp_tx-1][temp_ty][0]); }
	 		}
	}	
	
	// replace destination flag
	if (BuildMode==0)
		if ( (square_x!=sx) || (square_y!=sy) || (square_z!=sz) )
			if (Tiles[ square_x ][ square_y ][square_z]!=1) // tile is passable		// and <> start !!
			{
				// change dest. tile
				fx = square_x;
				fy = square_y;
				fz = square_z;
				// replace 
				var tempa = TileToCoord(fx,GridX);
				var tempb = TileToCoord(fy,GridY);
				me = destobject;
				my.x = tempa;
				my.y = tempb;
				AlignDestFlag();
			}
}


// rotates model and tile values too
function RotateEntity(tempx,tempy)
{
	you = ent_next(NULL); // get first entity
 	while (you) // repeat until there are no more entities
	{ 
		// check position stored in entity skills
		if ( (you.XTILE_NUMBER==tempx) && (you.YTILE_NUMBER==tempy) )
			if (you.ENTITY_TYPE < 3)
			{
//				ent_remove(you);  	
				you.pan += 90;		
				break;			// end of loop
			}				
 		you = ent_next(you); // get next entity
	}	
	
	if (you)
	if (you.AREA_SIZE>1)
	{
		// if a model with area size 3 e.g. a tower, rotate its tile values too
		var* temparray0 = NULL;
		var* temparray1 = NULL;
		var* temparray2 = NULL;
		var* temparrayZ0 = NULL;
		var* temparrayZ1 = NULL;
		var arraysize = you.AREA_SIZE * you.AREA_SIZE;
		var i,j,k;
		var tempa,tempb;
		if ((temparray2 == NULL) && (temparrayZ1 == NULL))
		{
			temparray0 = (var*)sys_malloc(arraysize*sizeof(var));	
			temparray1 = (var*)sys_malloc(arraysize*sizeof(var));	
			temparray2 = (var*)sys_malloc(arraysize*sizeof(var));	
			temparrayZ0 = (var*)sys_malloc(arraysize*sizeof(var));	
			temparrayZ1 = (var*)sys_malloc(arraysize*sizeof(var));		
			for(i=0;i<arraysize;i++)
			{
				temparray0[i] = 0;
				temparray1[i] = 0;
				temparray2[i] = 0;
				temparrayZ0[i] = 0;
				temparrayZ1[i] = 0;
			}
			tempa = you.XTILE_NUMBER - integer(you.AREA_SIZE/2);
			tempb = you.YTILE_NUMBER + integer(you.AREA_SIZE/2);			
			// LEVEL 0-1-2
			k=0;
			// increase column then decrease row
			for(j=0;j<you.AREA_SIZE;j++)
				for(i=0;i<you.AREA_SIZE;i++)
				{				
					if ( (tempa+i>=0) && (tempa+i<GridX) && (tempb-j>=0) && (tempb-j<GridY) )
					{
						temparray0[k] = Tiles[tempa+i][tempb-j][0];
						temparray1[k] = Tiles[tempa+i][tempb-j][1];
						temparray2[k] = Tiles[tempa+i][tempb-j][2];
						temparrayZ0[k] = TilesZ[tempa+i][tempb-j][0];
						temparrayZ1[k] = TilesZ[tempa+i][tempb-j][1];
					}
					else
					{
						temparray0[k] = 0;
						temparray1[k] = 0;
						temparray2[k] = 0;
						temparrayZ0[k] = 0;
						temparrayZ1[k] = 0;
					}
					k++;	
				}	
			tempa = you.XTILE_NUMBER - integer(you.AREA_SIZE/2);
			tempb = you.YTILE_NUMBER - integer(you.AREA_SIZE/2);
			k=0;
			// increase row then increase column
			for(i=0;i<you.AREA_SIZE;i++)
				for(j=0;j<you.AREA_SIZE;j++)			
				{	
					temp_tx = tempa + i;
					temp_ty = tempb + j;
					TileFill(temparray0[k],temparray1[k],temparray2[k],temparrayZ0[k],temparrayZ1[k]);
					k++;
				}
			
			sys_free(temparray0); 
			sys_free(temparray1); 
			sys_free(temparray2); 
			sys_free(temparrayZ0); 
			sys_free(temparrayZ1); 			
		}	
	}
}
