
///////////////////////////////////////////////////////////////////
// A* pathfinding function declarations - ending with 01
function PathFind01(heuristic);

function Path_Dots01();
function Create_Path_Array01();

function All_Path_Arrows01();
function DrawAllPaths01();
///////////////////////////////////////////////////////////////////

// A* pathfinder main function, including tile value conversion
function PathFind01(heuristic)
{
	// no collision detection needed for path entities
	enable_polycollision = 0; 
	
	// show a panel during calculation
	set( work_in_progress , SHOW );
	//work_in_progress.flags |= SHOW;  
	reset( path_found , SHOW );
	reset( path_error , SHOW );
	wait(5);
	
	time_1 = GetTickCount();
	
	////////// data initialization and path conversion from tile system //////////    
	var i,j,k, a,b,c;
	var error1;
	var tempx,tempy,tempz, tempa,tempb,tempc;
	var temptilez, temptilec;
	var test_cost;
	var multiplier1;        // multiply by: 1 if straight, 1.4 if diagonal move		
	var islevelchange;
	
	// start and finish tiles are already set, and stored in (sx,sy) and (fx,fy) respectively
	
	// a-star array nulling	
	for(k=0 ; k<VERTICAL_LEVELS ; k++)
		for(i=0 ; i<GridX ; i++)
			for(j=0 ; j<GridY ; j++)
			{						
				// possible values: 0 or 1
				astar_clist[i][j][k] = 0;
				astar_olist[i][j][k] = 0;
				// possible values: >=0
				astar_F[i][j][k] = 0;
				astar_G[i][j][k] = 0;
				astar_H[i][j][k] = 0;
				// possible values: 0..GridX/Y
				astar_parent_x[i][j][k] = -1;
				astar_parent_y[i][j][k] = -1;
				astar_parent_z[i][j][k] = -1;
				// earlier should be set, to make easier detection of path finding errors 
				// i.e. the start and finish should be on the same continent - no ship usage incuded
	//			astar_continent[i][j] = 1;
				// it should be removed later and give real values!!!
			}  	
	
	// Tile value conversion to pathfinding move cost values stored by DataArray
	for(k=0 ; k<VERTICAL_LEVELS ; k++)
		for(i=0 ; i<GridX ; i++)
			for(j=0 ; j<GridY ; j++)
			{						
				// water or other forbidden area - no access
				if (Tiles[i][j][k]%HEIGHT_OFFSET == 1) { DataArray[i][j][k] = 0; }
				
				// grass = normal terrain
				else if (integer(Tiles[i][j][k])%HEIGHT_OFFSET == 0) { DataArray[i][j][k] = GRASS_COST + fraction( Tiles[i][j][k] ); }
				// stone road
				else if (integer(Tiles[i][j][k])%HEIGHT_OFFSET == 2) { DataArray[i][j][k] = STONE_COST + fraction( Tiles[i][j][k] ); }
				// dirt road
				else if (integer(Tiles[i][j][k])%HEIGHT_OFFSET == 3) { DataArray[i][j][k] = DIRT_COST + fraction( Tiles[i][j][k] ); }
				// woods
				else if (integer(Tiles[i][j][k])%HEIGHT_OFFSET == 4) { DataArray[i][j][k] = WOODS_COST + fraction( Tiles[i][j][k] ); }
				// swamp
				else if (integer(Tiles[i][j][k])%HEIGHT_OFFSET == 7) { DataArray[i][j][k] = SWAMP_COST + fraction( Tiles[i][j][k] ); }
				
				// ladder
				else if (integer(Tiles[i][j][k])%HEIGHT_OFFSET == MODELS+24) { DataArray[i][j][k] = LADDER_COST + fraction( Tiles[i][j][k] ); }
				// stairs
				else if (integer(Tiles[i][j][k])%HEIGHT_OFFSET == MODELS+20) { DataArray[i][j][k] = STAIRS_COST + fraction( Tiles[i][j][k] ); }
				// on wall
				else if (integer(Tiles[i][j][k])%HEIGHT_OFFSET == MODELS+0) { DataArray[i][j][k] = ONWALL_COST + fraction( Tiles[i][j][k] ); }
				// in tower
				else if (integer(Tiles[i][j][k])%HEIGHT_OFFSET == MODELS+1) { DataArray[i][j][k] = ONWALL_COST + fraction( Tiles[i][j][k] ); }
				
				// if on ground, add slope movecost - mountain is double as hill
				if (integer(Tiles[i][j][k])%HEIGHT_OFFSET<MODELS) { DataArray[i][j][k] += integer(integer(Tiles[i][j][k])/HEIGHT_OFFSET)*HILL_COST; }
			}  	
	
	///////////// start tile settings ////////////////
	
	error1 = 0 ; // no error
	found = 0 ;	// finish not found  
	cycle_time = 0; // init cycle value
	maxpath = MAXPATHLENGTH * 100; // now it is max loop value - to avoid infinite cycle
	
// continent check
// if (astar_continent[sx][sy]==astar_continent[fx][fy])
// {
	
	// put start on open list
	astar_olist[sx][sy][sz] = 1;
	
	// H : give distance between start and finish, 
	// could be a bit fictious but should be proportional with G i.e. path distance walked
	// GRASS_COST is now the value of walk point decrease on grass, as an average value
	// so H and G will based on walk-point penalty values
	// astar_H[sx][sy] = integer( sqrt( pow( abs(sx - fx) *6 , 2 ) +  pow( abs(sy - fy) *6 , 2 ) ) );
	
	// H : estimated distance, or "heuristic" function: 	
	if (heuristic == 1)
	{	
		// MANHATTAN value = ( x-distance + y-distance ) * weight (e.g. an average cost value)
		astar_H[sx][sy][sz] = integer( ( (abs(sx - fx)) + (abs(sy - fy)) ) * heur_value );						
	}
	else if (heuristic == 2)
	{
		// EUCLIDIAN value = sqrt( sqr(x-distance) + sqr(y-distance) ) * weight (e.g. an average cost value)
		astar_H[sx][sy][sz] = integer( sqrt(pow(abs(sx - fx),2) + pow(abs(sy - fy),2)) * heur_value );
	}
	else if (heuristic == 3)
	{
		// DIAGONAL SHORTCUT value : diagonal distance * 1.4 until fx or fy reached + the remainig orthogonal distance  
		astar_H[sx][sy][sz] = integer( ( (minv( (abs(sx - fx)) , (abs(sy - fy)) ))*1.4 + abs( (abs(sx - fx)) - (abs(sy - fy)) ) ) * heur_value );	
	}
	else astar_H[sx][sy][sz] = 0; // Dijktsra        
	
	// G : path length completed
	astar_G[sx][sy][sz] = 0;
	// F : weighting value
	astar_F[sx][sy][sz] = astar_H[sx][sy][sz] + astar_G[sx][sy][sz];
	
	cycle_time++;  // just for checking
	
	////////// the main loop ///////////////

	while ((!found) && (!error1))
	{
		cycle_time++;
		
		// big value needed for finding the tile having the smallest F value
		a = 10000;
		// tempa and tempb show the tile having the smallest F value
		tempa = -1;		
		tempb = -1;
		tempc = -1;
		// b is >0 if there exists an open list element
		b = 0;
		
		for(k=0 ; k<VERTICAL_LEVELS ; k++)
			for(j=0 ; j<GridY ; j++)			
				for(i=0 ; i<GridX ; i++)
				{
					if (astar_olist[i][j][k]>0) b++;
					
					// looking for open list element having the smallest value
					if ((astar_olist[i][j][k]>0) && (astar_F[i][j][k]<=a))
					{
						a = astar_F[i][j][k]	;
						// tempa-b stores this array position
						tempa = i;
						tempb = j;
						tempc = k;
					}	
				}
		// now tile [tempa][tempb] holds the best position, and its neighbours will be checked now
		
		// if no open list element , it means error
		if (b == 0) error1++;
		
		// another error situation, if no smallest F value tile determined
		if ((b == 0) && ((tempa<0) || (tempb<0) || (tempc<0))) error1++;
		
		// the smallest element goes to closed list from open list
		astar_olist[tempa][tempb][tempc] = 0;
		astar_clist[tempa][tempb][tempc] = 1;
		
		// looking around, i.e. checking the 8 possible neighbours
		// with diagonal travel penalty		
		i = 0;
		while ((i<10) && (!found) && (!error1))
		{
			i++;
			islevelchange = 0;
			
			// tempx,tempy,tempz will hold the array position to be examined
			if (i<9) { tempz = tempc; }
			// S
			if (i == 1)
			{
				tempx = tempa ;
				tempy = tempb -1;
				multiplier1 = 1;
			}
			// E
			else if (i == 2)
			{
				tempx = tempa +1;
				tempy = tempb ;
				multiplier1 = 1;
			}
			// N
			else if (i == 3)
			{
				tempx = tempa ;
				tempy = tempb +1;
				multiplier1 = 1;
			}
			// W
			else if (i == 4)
			{
				tempx = tempa -1;
				tempy = tempb ;
				multiplier1 = 1;
			}			
			// SW
			else if (i == 5)
			{
				tempx = tempa -1;
				tempy = tempb -1;
				multiplier1 = 1.4;
			}
			// SE
			else if (i == 6)
			{
				tempx = tempa +1;
				tempy = tempb -1;
				multiplier1 = 1.4;
			}
			// NE
			else if (i == 7)
			{
				tempx = tempa +1;
				tempy = tempb +1;
				multiplier1 = 1.4;
			}
			// NW
			else if (i == 8)
			{
				tempx = tempa -1;
				tempy = tempb +1;
				multiplier1 = 1.4;
			}			
			
			// check level	change
			// previous tile should have a level change towards this tile, and this tile backwards too
			// so level change direction is mutually determined by the pre-processed map			
			else if (i == 9)
			{
				// there are 8 possible cases, 4 upwards and 4 downwards in straight directions
				if ( fraction(DataArray[tempa][tempb][tempc]) > 0 )
				{
					multiplier1 = 1;		// no need to set it higher because to decrease the chance of unnecessary level changes is done by move cost settings
					
					// S.U 0.1/0.35
					if ( ( fraction(DataArray[ tempa ][ tempb ][ tempc ]) > 0.08 ) && ( fraction(DataArray[ tempa ][ tempb ][ tempc ]) < 0.12 ) )
					{
//						if ( ( fraction(DataArray[ tempa ][ tempb-1 ][ tempc+1 ]) > 0.33 ) && ( fraction(DataArray[ tempa ][ tempb-1 ][ tempc+1 ]) < 0.37 ) )
						{
							tempx = tempa ;
							tempy = tempb - 1;
							tempz = tempc + 1;
							islevelchange = 1;
						}
					}
					// E.U 0.2/0.45
					else if ( ( fraction(DataArray[ tempa ][ tempb ][ tempc ]) > 0.18 ) && ( fraction(DataArray[ tempa ][ tempb ][ tempc ]) < 0.22 ) )
					{
//						if ( ( fraction(DataArray[ tempa+1 ][ tempb ][ tempc+1 ]) > 0.43 ) && ( fraction(DataArray[ tempa+1 ][ tempb ][ tempc+1 ]) < 0.47 ) )
						{
							tempx = tempa + 1;
							tempy = tempb ;
							tempz = tempc + 1;
							islevelchange = 1;
						}
					}
					// N.U 0.3/0.15
					else if ( ( fraction(DataArray[ tempa ][ tempb ][ tempc ]) > 0.28 ) && ( fraction(DataArray[ tempa ][ tempb ][ tempc ]) < 0.32 ) )
					{
//						if ( ( fraction(DataArray[ tempa ][ tempb+1 ][ tempc+1 ]) > 0.13 ) && ( fraction(DataArray[ tempa ][ tempb+1 ][ tempc+1 ]) < 0.17 ) )
						{
							tempx = tempa ;
							tempy = tempb + 1;
							tempz = tempc + 1;
							islevelchange = 1;
						}
					}
					// W.U 0.4/0.25
					else if ( ( fraction(DataArray[ tempa ][ tempb ][ tempc ]) > 0.38 ) && ( fraction(DataArray[ tempa ][ tempb ][ tempc ]) < 0.42 ) )
					{
//						if ( ( fraction(DataArray[ tempa-1 ][ tempb ][ tempc+1 ]) > 0.23 ) && ( fraction(DataArray[ tempa-1 ][ tempb ][ tempc+1 ]) < 0.27 ) )
						{
							tempx = tempa - 1;
							tempy = tempb ;
							tempz = tempc + 1;
							islevelchange = 1;
						}
					}
					// S.D 0.15/0.3
					else if ( ( fraction(DataArray[ tempa ][ tempb ][ tempc ]) > 0.13 ) && ( fraction(DataArray[ tempa ][ tempb ][ tempc ]) < 0.17 ) )
					{
//						if ( ( fraction(DataArray[ tempa ][ tempb-1 ][ tempc-1 ]) > 0.28 ) && ( fraction(DataArray[ tempa ][ tempb-1 ][ tempc-1 ]) < 0.32 ) )
						{
							tempx = tempa ;
							tempy = tempb - 1;
							tempz = tempc - 1;
							islevelchange = 1;
						}
					}
					// E.D 0.25/0.4
					else if ( ( fraction(DataArray[ tempa ][ tempb ][ tempc ]) > 0.23 ) && ( fraction(DataArray[ tempa ][ tempb ][ tempc ]) < 0.27 ) )
					{
//						if ( ( fraction(DataArray[ tempa+1 ][ tempb ][ tempc-1 ]) > 0.38 ) && ( fraction(DataArray[ tempa+1 ][ tempb ][ tempc-1 ]) < 0.42 ) )
						{
							tempx = tempa + 1;
							tempy = tempb ;
							tempz = tempc - 1;
							islevelchange = 1;
						}
					}
					// N.D 0.35/0.1
					else if ( ( fraction(DataArray[ tempa ][ tempb ][ tempc ]) > 0.33 ) && ( fraction(DataArray[ tempa ][ tempb ][ tempc ]) < 0.37 ) )
					{
//						if ( ( fraction(DataArray[ tempa ][ tempb+1 ][ tempc-1 ]) > 0.08 ) && ( fraction(DataArray[ tempa ][ tempb+1 ][ tempc-1 ]) < 0.12 ) )
						{
							tempx = tempa ;
							tempy = tempb + 1;
							tempz = tempc - 1;
							islevelchange = 1;
						}
					}
					// W.D 0.45/0.2
					else if ( ( fraction(DataArray[ tempa ][ tempb ][ tempc ]) > 0.43 ) && ( fraction(DataArray[ tempa ][ tempb ][ tempc ]) < 0.47 ) )
					{
//						if ( ( fraction(DataArray[ tempa-1 ][ tempb ][ tempc-1 ]) > 0.18 ) && ( fraction(DataArray[ tempa-1 ][ tempb ][ tempc-1 ]) < 0.22 ) )
						{
							tempx = tempa - 1;
							tempy = tempb ;
							tempz = tempc - 1;
							islevelchange = 1;
						}
					}
					else
					{
						islevelchange = 0;
					}
				}				
			}
			
			// TileZ has less dimensoins by 1, so needs a check
			temptilec = tempc - 1;
			if (temptilec<0) { temptilec = 0; }
			temptilez = tempz - 1;
			if (temptilez<0) { temptilez = 0; }
			
			// now tempa-b-c stores the original tile, and tempx-y-z the neighbour examined actually
						
			// needs a safety check of array borders
			if ((tempx >-1) && (tempy>-1) && (tempz>-1) && (tempx<GridX) && (tempy<GridY) && (tempz<VERTICAL_LEVELS))			
			// AND neighbour should be accessible, and not on closed list
			if (( integer(DataArray[tempx][tempy][tempz]) > 0 ) && (!astar_clist[tempx][tempy][tempz]))			
			// AND should be within achievable height 
			// but height difference is not allowed in diagonal directions to keep movement nice
			// moreover level change does not require TileZ check because of pre-processed map structure
			if ( ( (i<5) && (abs( TilesZ[tempa][tempb][temptilec] - TilesZ[tempx][tempy][temptilez] ) < MAX_HEIGHT_DIFFERENCE) )
			  || ( (i>4) && (i<9) && (abs(TilesZ[tempa][tempb][temptilec] - TilesZ[tempx][tempy][temptilez]) == 0 ) )
			  || ( (i>8) && (islevelchange>0) ) ) 
			{
				if (astar_olist[tempx][tempy][tempz])	   // if tile is already on open list
				{
					// check whether now G is smaller than previously i.e. now the path is shorter to here
					// dataarray stores move cost terrain properties
						
					if ( (astar_G[tempa][tempb][tempc] + integer(DataArray[tempx][tempy][tempz])) < astar_G[tempx][tempy][tempz]  )
					{
						// giving new values to F G and "parent" in this tile 
						
						// G = parent value + (current terrain penalty * direction multiplier1) + height penalty (0 because not used)
						astar_G[tempx][tempy][tempz] = astar_G[tempa][tempb][tempc] + (integer(DataArray[tempx][tempy][tempz])*multiplier1) + 0 ;  
						// H value must exist because of previous examination is done
						astar_F[tempx][tempy][tempz] = astar_G[tempx][tempy][tempz] + astar_H[tempx][tempy][tempz]; 
						
						astar_parent_x[tempx][tempy][tempz] = tempa;
						astar_parent_y[tempx][tempy][tempz] = tempb;
						astar_parent_z[tempx][tempy][tempz] = tempc;
					}
				}
				else 		// if tile is not on open list yet
				{
					astar_olist[tempx][tempy][tempz] = 1;
					
					astar_G[tempx][tempy][tempz] = astar_G[tempa][tempb][tempc] + (integer(DataArray[tempx][tempy][tempz])*multiplier1) + 0 ;  
					
					if (heuristic == 1)
					{
						// MANHATTAN
						astar_H[tempx][tempy][tempz] = integer( ( (abs(tempx - fx)) + (abs(tempy - fy)) ) * heur_value );
					}
					else if (heuristic == 2)
					{
						// EUCLIDIAN
						astar_H[tempx][tempy][tempz] = integer( sqrt(pow(abs(tempx - fx),2) + pow(abs(tempy - fy),2)) * heur_value );
					}
					else if (heuristic == 3)
					{
						// DIAGONAL SHORTCUT
						astar_H[tempx][tempy][tempz] = integer( ( (minv( (abs(tempx - fx)) , (abs(tempy - fy)) ))*1.4 + abs( (abs(tempx - fx)) - (abs(tempy - fy)) ) ) * heur_value );
					}
					else astar_H[tempx][tempy][tempz] = 0; // Dijktsra   
					
					// depth first
					if ( (depthfirst) && (heuristic>0) )
						astar_G[tempx][tempy] = 0;
						
					astar_F[tempx][tempy][tempz] = astar_G[tempx][tempy][tempz] + astar_H[tempx][tempy][tempz] ;
					
					astar_parent_x[tempx][tempy][tempz] = tempa;
					astar_parent_y[tempx][tempy][tempz] = tempb;
					astar_parent_z[tempx][tempy][tempz] = tempc;
				}				
			}			
		}		// while 1..8 direction check
		
		// if finish point is found
		if (astar_olist[fx][fy][fz]) found++;
		
		// to avoid infinite loop, exit while loop
		if ( cycle_time >= maxpath ) error1++; 
		
	}	// while
	
	///// error message 
	if ((found<1) || (error1>0))
	{
		set( path_error , SHOW );	
		
		pathlength = 0;
		total_move_cost = 0;
		
		if (drawalltiles) DrawAllPaths01(); // to see where calcucation failed
		
		wait(1);
		wait(-2);
		reset( path_error , SHOW );
		
		EndOfPathfinding(); // allow to do something again		
	}
	
//  } // continent check
//  else
//  {
//  		set( path_error2 , SHOW );	// should be a different error message, to be followed by creating a combined ground+sea path 
//  }	
		
	///// creating path array , showing 2D array positions
	if (found>0)
	{
		// found panel
		set( path_found , SHOW );		
		// create path array and show path 
		Create_Path_Array01();			
	}   // found>0
	
	// remove panel
	wait(1);
	reset( work_in_progress , SHOW );
	
	// reset collision detection
	enable_polycollision = 1; 
}


//// ------------------------------------

// setting path dot properties
function Path_Dots01()
{
	// tile coordinates
	var tempx = my.x;
	var tempy = my.y;
	var tempz = my.z;	
	// horizintal alignment
	vec_set( my.pan , vector(270,90,0) );       	
	// scale to actual tilesize
	my.scale_x = TileSize/GRIDSIZE;
	my.scale_y = TileSize/GRIDSIZE;
	// convert tile coordinates to real coordinates
	my.x = TileToCoord( tempx , GridX );
	my.y = TileToCoord( tempy , GridY );	
	SnapIt();
	
	if (tempz==2)				
	{ my.z = TilesMin[tempx][tempy] + PathLevel + (TilesZ[tempx][tempy][0]+TilesZ[tempx][tempy][1])*TileSize; }
	else if (tempz==1)
	{ my.z = TilesMin[tempx][tempy] + PathLevel + TilesZ[tempx][tempy][0]*TileSize; }
	else 
	{ my.z = TilesMax[tempx][tempy] + PathLevel; }	
	
	set( me , PASSABLE | ZNEAR );		// | ZNEAR - to check hidden tiles
	my.ambient = 100;  
	my.emask &= ~DYNAMIC;
}


// could be included into main pathfinding function 
function Create_Path_Array01()
{
	var i,j, k, tempx,tempy,tempz, tempd, counter,counterm;
	
	//// clearing existing path data ////
	for(i=0 ; i<MAXPATHLENGTH ; i++)
	{
		// pathX and Y store x and y array-coordinates within the range 0..999, so 1000 is the value of no path elements
		pathX[i] = 1000;
		pathY[i] = 1000;
		pathZ[i] = 1000;
		// pathDir can show in 8 directions or nowhere=0 . it is needed for representing the path by small arrows
		pathDir[i] = 0;
	}
	
	// temporarily the first element of path-array is the finish node
	counter = 0;			
	pathX[counter] = fx;
	pathY[counter] = fy; 
	pathZ[counter] = fz; 
	pathDir[counter] = 1; // final direction can be any, but should be added now to keep array number - set later
	maxpath = MAXPATHLENGTH;     // now it shows the global maximum value given 
	
	///// looking for start position backward from finish /////
	while ( ((pathX[counter] != sx) || (pathY[counter] != sy) || (pathZ[counter] != sz)) && (counter<(maxpath-1)) )
	//for(i=0 ; (i<(maxpath-2)) ; i++ )
	{
		// counter now points at the current element
		counter += 1;
		counterm = counter - 1;
		
		// pointing at the previous elment's position
		tempx = pathX[counterm] ;
		tempy = pathY[counterm] ;
		tempz = pathZ[counterm] ;
		
		if ((astar_parent_x[ tempx ][ tempy ][ tempz ] < 1000) 
		&& (astar_parent_y[ tempx ][ tempy ][ tempz ] < 1000)
		&& (astar_parent_z[ tempx ][ tempy ][ tempz ] < 1000))   
		{
			// the new element is the parent of the previous element
			pathX[counter] = astar_parent_x[ tempx ][ tempy ][ tempz ] ;
			pathY[counter] = astar_parent_y[ tempx ][ tempy ][ tempz ] ;
			pathZ[counter] = astar_parent_z[ tempx ][ tempy ][ tempz ] ;
			
			// horizontal path direction - now current and previous elements are available to examine their direction
			if ( (pathX[counter]==pathX[counterm]) && (pathY[counter]-1==pathY[counterm]) )
			{ pathDir[counter] = 1; }
			else if ( (pathX[counter]+1==pathX[counterm]) && (pathY[counter]==pathY[counterm]) )
			{ pathDir[counter] = 2; }
			else if ( (pathX[counter]==pathX[counterm]) && (pathY[counter]+1==pathY[counterm]) )
			{ pathDir[counter] = 3; }
			else if ( (pathX[counter]-1==pathX[counterm]) && (pathY[counter]==pathY[counterm]) )
			{ pathDir[counter] = 4; }
			else if ( (pathX[counter]-1==pathX[counterm]) && (pathY[counter]-1==pathY[counterm]) )
			{ pathDir[counter] = 5; }
			else if ( (pathX[counter]+1==pathX[counterm]) && (pathY[counter]-1==pathY[counterm]) )
			{ pathDir[counter] = 6; }
			else if ( (pathX[counter]+1==pathX[counterm]) && (pathY[counter]+1==pathY[counterm]) )
			{ pathDir[counter] = 7; }
			else if ( (pathX[counter]-1==pathX[counterm]) && (pathY[counter]+1==pathY[counterm]) )
			{ pathDir[counter] = 8; }
		}				
		// set first elements direction again - final path step shows now in the same direction as previous step
		pathDir[0] = pathDir[1];		
	} // while
	
	// error - too long path
	if (counter > (maxpath - 2))
		set( path_error , SHOW );
	
	// now it shows the real path length
	maxpath = counter +1;	
	
	////// finalizing the path //////
	tempx = 0;
	tempy = 0;
	tempz = 0;
	tempd = 0;
	var maxpathlength_minus1_minusi;
	for(i=0 ; i<maxpath ; i++)               
	{
		maxpathlength_minus1_minusi = maxpath -1 -i ;
		// last element to temp
		tempx = pathX[ maxpathlength_minus1_minusi ] ;
		tempy = pathY[ maxpathlength_minus1_minusi ] ;
		tempz = pathZ[ maxpathlength_minus1_minusi ] ;
		tempd = pathDir[ maxpathlength_minus1_minusi ];
		// first element to last
		pathX[ maxpathlength_minus1_minusi ] = pathX[ i ];
		pathY[ maxpathlength_minus1_minusi ] = pathY[ i ];
		pathZ[ maxpathlength_minus1_minusi ] = pathZ[ i ];
		pathDir[ maxpathlength_minus1_minusi ] = pathDir[ i ];
		// temp element to first
		pathX[ i ] = tempx;
		pathY[ i ] = tempy;
		pathZ[ i ] = tempz;
		pathDir[ i ] = tempd;
	}
	// pathX and Y are ready to use		
	
	// get time length of path calculation, and without path display
	time_2 = GetTickCount();
	time_elapsed = (time_2 - time_1)/1000;
	str_for_num(timer_value, time_elapsed);
	
	///// representing the path //////
	if (drawalltiles)
	{
		k = 0;
		for( i=0 ; i<maxpath ; i++ )
		{
			if ( (pathX[i] < 1000) && (pathY[i] < 1000) && (pathZ[i] < 1000) )
			{   		 						
				PathArray[k] = ent_create( path_dot , vector(pathX[i],pathY[i],pathZ[i]) , Path_Dots01 );			
				k++;
			}
		}	
	}
	else
	{
		k = 0;
		for( i=0 ; i<maxpath ; i++ )
		{
			if ( (pathX[i] < 1000) && (pathY[i] < 1000) && (pathZ[i] < 1000) )
			{   		 			
				if (pathDir[i]==1)		{ PathArray[k] = ent_create( "1.tga" , vector(pathX[i],pathY[i],pathZ[i]) , Path_Dots00 );	 }
				else if (pathDir[i]==2)	{ PathArray[k] = ent_create( "2.tga" , vector(pathX[i],pathY[i],pathZ[i]) , Path_Dots00 );	 }
				else if (pathDir[i]==3)	{ PathArray[k] = ent_create( "3.tga" , vector(pathX[i],pathY[i],pathZ[i]) , Path_Dots00 );	 }
				else if (pathDir[i]==4)	{ PathArray[k] = ent_create( "4.tga" , vector(pathX[i],pathY[i],pathZ[i]) , Path_Dots00 );	 }
				else if (pathDir[i]==5)	{ PathArray[k] = ent_create( "5.tga" , vector(pathX[i],pathY[i],pathZ[i]) , Path_Dots00 );	 }
				else if (pathDir[i]==6)	{ PathArray[k] = ent_create( "6.tga" , vector(pathX[i],pathY[i],pathZ[i]) , Path_Dots00 );	 }
				else if (pathDir[i]==7)	{ PathArray[k] = ent_create( "7.tga" , vector(pathX[i],pathY[i],pathZ[i]) , Path_Dots00 );	 }
				else if (pathDir[i]==8)	{ PathArray[k] = ent_create( "8.tga" , vector(pathX[i],pathY[i],pathZ[i]) , Path_Dots00 );	 }
				else 							{ PathArray[k] = ent_create( path_dot , vector(pathX[i],pathY[i],pathZ[i]) , Path_Dots00 );	}		// error
				k++;
			}
		}	
	}
	
	// display path length
	pathlength = maxpath;
	
	// representing all search values, just for fun
	if (drawalltiles) DrawAllPaths01();
	
	// removing the little path dots
	wait(1);
	wait(-10);
	reset( path_found , SHOW );
	
	for(i=0 ; i<k ; i++)
		ptr_remove( PathArray[i] );
	wait(1);
	
	// get path cost
	if (!drawalltiles)
	{
		Path_Cost_Calculation();
		EndOfPathfinding(); // allow to do something again		
	}
}


////-----------------------------------------

// setting direction signs' position
function All_Path_Arrows01()
{
	// tile coordinates
	var tempx = my.x;
	var tempy = my.y;	
	var tempz = my.z;	
	// horizintal alignment
	vec_set( my.pan , vector(270,90,0) );       	
	// scale to actual tilesize
	my.scale_x = TileSize/GRIDSIZE;
	my.scale_y = TileSize/GRIDSIZE;
	// convert tile coordinates to real coordinates	
	my.x = TileToCoord( tempx , GridX );
	my.y = TileToCoord( tempy , GridY );	
	SnapIt();
	
	// set z
//	my.z = TilesMax[tempx][tempy] + PathLevel + RoadLevel + (tempz*TileSize*3); // constant height on each level - can be useful with ZNEAR in case of error checking

	if (tempz==2)				
	{ my.z = TilesMax[tempx][tempy] + PathLevel + RoadLevel + (TilesZ[tempx][tempy][0]+TilesZ[tempx][tempy][1])*TileSize; }
	else if (tempz==1)
	{ my.z = TilesMax[tempx][tempy] + PathLevel + RoadLevel + TilesZ[tempx][tempy][0]*TileSize; }
	else 
	{ my.z = TilesMax[tempx][tempy] + PathLevel + RoadLevel; }	
	
	set( me , PASSABLE );		// | ZNEAR
	my.ambient = 100;
	my.emask &= ~DYNAMIC;
}


// drawing directions of parent tiles - each tile arrow shows where it came from - not as path, where to go
function DrawAllPaths01()
{
	var i,j,m, k;
	var tempx,tempy,tempz, tempa,tempb,tempc;
	
	for(m=0 ; m<VERTICAL_LEVELS; m++)		// represent arrow arrays level by level
	{
		k = 0; // entity array counter
		for(j=0 ; j<GridY; j++)
			for(i=0 ; i<GridX; i++)
			{			
				// if accessible
				if ( DataArray[i][j][m] > 0 )
				{
					// if not start - only start has no parent so finish can be drawn with direction arrow too
					if (( i!= sx ) || ( j!= sy ) || ( k!= sz )) 
					{
						// no value calculated  =-1
						if ((astar_parent_x[i][j][m] < 0) && (astar_parent_y[i][j][m] < 0) && (astar_parent_z[i][j][m] < 0) )
						{ 
							ArrowArray[k] = ent_create( "0.tga" , vector(i,j,m) , All_Path_Arrows01 ); 
						}	
						else
						{
							// horizontal direction values
							if ((astar_parent_x[i][j][m] == (i)) && (astar_parent_y[i][j][m] == (j+1)))
								{ ArrowArray[k] = ent_create( "1.tga" , vector(i,j,m) , All_Path_Arrows01 ); }
							else if ((astar_parent_x[i][j][m] == (i-1)) && (astar_parent_y[i][j][m] == (j)))
								{ ArrowArray[k] = ent_create( "2.tga" , vector(i,j,m) , All_Path_Arrows01 ); }
							else if ((astar_parent_x[i][j][m] == (i)) && (astar_parent_y[i][j][m] == (j-1)))
								{ ArrowArray[k] = ent_create( "3.tga" , vector(i,j,m) , All_Path_Arrows01 ); }
							else if ((astar_parent_x[i][j][m] == (i+1)) && (astar_parent_y[i][j][m] == (j)))
								{ ArrowArray[k] = ent_create( "4.tga" , vector(i,j,m) , All_Path_Arrows01 ); }
							
							else if ((astar_parent_x[i][j][m] == (i+1)) && (astar_parent_y[i][j][m] == (j+1)))
								{ ArrowArray[k] = ent_create( "5.tga" , vector(i,j,m) , All_Path_Arrows01 ); }
							else if ((astar_parent_x[i][j][m] == (i-1)) && (astar_parent_y[i][j][m] == (j+1)))
								{ ArrowArray[k] = ent_create( "6.tga" , vector(i,j,m) , All_Path_Arrows01 ); }
							else if ((astar_parent_x[i][j][m] == (i-1)) && (astar_parent_y[i][j][m] == (j-1)))
								{ ArrowArray[k] = ent_create( "7.tga" , vector(i,j,m) , All_Path_Arrows01 ); }
							else if ((astar_parent_x[i][j][m] == (i+1)) && (astar_parent_y[i][j][m] == (j-1)))
								{ ArrowArray[k] = ent_create( "8.tga" , vector(i,j,m) , All_Path_Arrows01 ); }
						} // else
					}	// not start position						
				} // accessible
				else				
		//		if ( DataArray[i][j] == 0 )		// if no access
				{ 
					ArrowArray[k] = ent_create( "11.tga" , vector(i,j,m) , All_Path_Arrows01 ); 
				}
					
				k++;
			}	// for i j	
		
		// start and end positions can be drawn now
		ArrowArray[k] = ent_create( "10.tga" , vector(fx,fy,fz) , All_Path_Arrows01 );
		k++;
		ArrowArray[k] = ent_create( "9.tga" , vector(sx,sy,sz) , All_Path_Arrows01 );
		k++;
		
		// removing the little arrows showing all possible directions
		wait(1);
		wait(-3);
		for(i=0 ; i<k ; i++)
			ptr_remove( ArrowArray[i] );		
	}	// for m
	
	Path_Cost_Calculation();
	EndOfPathfinding(); // allow to do something again		
}


